----------------------------------------------------------
-- MuffinFactionizer, a reputation management tool  --
-------------------------------------------------------
---------------------------
-- _00_ Variables Set up --
---------------------------

local ADDON_NAME, MFAC = ... -- Pulls back the Addon-Local Variables and store them locally.

--TODO: Either implement this or remove it
MFAC.ToBFF = {}	--> Friendship levels:
MFAC.ToBFF[0] = 42999;	--> 1 - Stranger: 0-8400
MFAC.ToBFF[1] = 42000;	--> 2 - Acquaintance: 8400-16800
MFAC.ToBFF[2] = 33600;	--> 3 - Buddy: 16800-25200
MFAC.ToBFF[3] = 25200;	--> 4 - Friend: 25200-33600
MFAC.ToBFF[4] = 16800;	--> 5 - Good Friend: 33600-42000
MFAC.ToBFF[5] = 8400;	--> 6 - Best Friend: 42000-42999

--------------------------
-- _01_ Addon Variables --
--------------------------

-- Initialization
MFAC.MainWindow = nil   -- Main program window

-- Faction information
MFAC.FactionIDToName = {}
MFAC.FactionNameToID = {}

local UI = MFAC.UI


FIZ_FactionGain = {} --Central database of all ways to gain rep.  TODO: Document it's structure
--		.[faction_id]  	The id of the faction
--			.[standing]		The standing id (1-8) with the faction
--				.quests
--					.count				The highest index in the quest array (not actually a count)
--					.data					The list of quest data:
--						.name 			The quest name
--						.rep				Amount of rep earned
--						.maxStanding	Max standing (1-8)
--						.profession		Required tradeskills
--						.items			Key/Value: Item Name/# of items required

-- Caching previous rep values to detect changes
MFAC.StoredRep = {}

--- Race/Side
MFAC.IsHuman = false
MFAC.IsDeathKnight = false
MFAC.IsAlliance = false
MFAC.IsHorde = false

-- Guild Tracking
MFAC.GuildName = nil


------------------------
-- _02_ Addon Startup --
------------------------
------------------------------------------------------------


MFAC.preloads = {
	146935, -- Valarjar (BoP)
	146941, -- Valarjar (BoA)
	150925, -- Greater Valarjar (BoA)

	146936, -- Dreamweaver Insignia (BoP)
	146942, -- Dreamweaver Insignia (BoA)

	146944, -- Highmountain Tribe Insignia (BoA)
	150928, -- Greater Highmountain (BoA)

	146939, -- Warden's Insignia (BoP)
	146945, -- Wardens Insignia (BoA)
	159929, -- Greater Wardens Insignia (BoA)

	146940, -- Nightfallen Insignia (BoP)
	146949, -- Legionfall Insignia
	146950, -- Legionfall Insignia (BoA)
	150924, -- Greater Tribute of the Broken Isles (BoP)
	152958, -- Army of the Light Insignia (BoP)
	152957, -- Army of the Light Insignia (BoA)

	128315, -- Medallion of the Legion
};

function MFAC.preload_rep_tokens()

	for _, value in ipairs(MFAC.preloads) do
		C_Item.RequestLoadItemDataByID(value)
	end

end

function MFAC.preload_reward_quests()

	for quest_id, _ in ipairs(MuffinFactionizer.paragon_supply_quests) do
		C_QuestLog.GetQuestInfo(quest_id)
		C_QuestLog.GetQuestObjectives(quest_id)
	end
	for quest_id, _ in ipairs(MuffinFactionizer.emissary_reward_quests) do
		C_QuestLog.GetQuestInfo(quest_id)
		C_QuestLog.GetQuestObjectives(quest_id)
	end

end


local function check_emissary_completion()
local quests = MFAC.emissary_quest_checks or {}
local remove_list = {}
	for quest_id, _v in pairs(quests) do
		local faction_id = MuffinFactionizer.emissary_reward_quests[quest_id];
		if (faction_id) then
			local faction_name = GetFactionInfoByID(faction_id);
			if (MFAC.is_quest_ready_for_turnin(quest_id)) then
				local icon_id = MuffinFactionizer.faction_toast_icons[faction_id]
				MFAC.ToastFakeAchievement(faction_name, icon_id, -1, "Emissary Reward Available");
				remove_list[quest_id] = quest_id
			end
		end
	end

	for k, _v in pairs(remove_list) do
		quests[k] = nil
	end

end
------------------------
-- _03_ Event Handler --
------------------------
--local last_event
function FIZ_OnEvent(_self, event, ...)
	local debug_start = debugprofilestop();

	local arg1, arg2 = ...
	if (event == "UPDATE_FACTION" or event == "QUEST_TURNED_IN" or event == "QUEST_WATCH_UPDATE") then
		if ( ReputationFrame:IsVisible() ) then
			ReputationFrame_Update();
		end
		if ( UI.ReputationDetailFrame:IsVisible()) then
			FIZ_BuildUpdateList()
			MFAC.UpdateDetailList()
		end
		FIZ_DumpReputationChangesToChat()

	elseif ( event == "BAG_UPDATE_DELAYED") then
		if (UI.ReputationDetailFrame:IsVisible()) then
			-- Update rep frame (implicitly updates detail frame which in turn implicitly reparses bag contents)
			ReputationFrame_Update()
		end

	elseif ( event == "BANKFRAME_OPENED") then
		FIZ_BankOpen = true
	elseif ( event == "BANKFRAME_CLOSED") then
		-- this is fired twice when closing the bank window, bank contents only available at the first Event
		if (FIZ_BankOpen) then
			-- this is the first call
			MFAC.ParseBankContent()
			FIZ_BankOpen = nil

			if (UI.ReputationDetailFrame:IsVisible()) then
				-- Update rep frame (implicitely updates detail frame which In turn implicitely reparses bag contents)
				ReputationFrame_Update()
			end
		end
	elseif (event == "GET_ITEM_INFO_RECEIVED" or event == "ITEM_DATA_LOAD_RESULT") then  --Item Info arrived and it was one we were missing
		if(MFAC.CacheMiss[arg1]) then
			MFAC_Data.ItemNameCache[arg1] = GetItemInfo(arg1);
			MFAC.CacheMiss[arg1] = nil;
			if (MFAC.is_table_empty(MFAC.CacheMiss)) then
				MFAC.MainWindow:UnregisterEvent("GET_ITEM_INFO_RECEIVED")
				MFAC.MainWindow:UnregisterEvent("ITEM_DATA_LOAD_RESULT")
			end
		end
	elseif (event == "QUEST_ACCEPTED") then
		--print(event, arg1, arg2, arg3);
		if(MFAC_Data.ShowToasts) then
			local quest_id = arg2
			local faction_id = MuffinFactionizer.paragon_supply_quests[quest_id];
			if (faction_id) then
				local faction_name = GetFactionInfoByID(faction_id);
				local icon_id = MuffinFactionizer.faction_toast_icons[faction_id]
				MFAC.ToastFakeAchievement(faction_name, icon_id, -1, "Paragon Reward Available");
			end
		end
	elseif ( event == "CHAT_MSG_SKILL") or
		( event == "TRADE_SKILL_LIST_UPDATE") or
		( event == "SKILL_LINES_CHANGED") or
		( event == "UPDATE_TRADESKILL_RECAST") then
		MFAC.ExtractSkills()
		if ( ReputationFrame:IsVisible() ) then
			ReputationFrame_Update(); -- rfl Event
		end
		if ( UI.ReputationDetailFrame:IsVisible()) then
			FIZ_BuildUpdateList()
			MFAC.UpdateDetailList()
		end
	elseif (event == "GOSSIP_SHOW" and MFAC_Data.AutoTurnIn) then
		local npc_id = MFAC.get_npc_id(UnitGUID("npc"));
		local npc_info = MuffinFactionizer.TurnInNPC[npc_id];
		local is_known = npc_info ~= nil;
		if(is_known) then
			local avail_quest_count = GetNumGossipAvailableQuests();
			if(MFAC_Data.LogAutoTurnin) then
				print("Gossip_Show:");
				print("   NPC ID:", npc_id, "IsKnown:", is_known);
				--print("   Faction:", npc_info.faction_id)
				--print("   GetNumGossipOptions:", GetNumGossipOptions())
				--print("   GetNumGossipActiveQuests:", GetNumGossipActiveQuests())
				print("   GetNumGossipAvailableQuests:", avail_quest_count)
			end
			local raw_data = {GetGossipAvailableQuests()};
			for i = 1, avail_quest_count do
				local base = ((i-1) * 7) + 1;
				local quest_name = raw_data[base];
				local can_handle = npc_info.quest_names[quest_name] ~= nil; -- Is this a quest we know how to handle?
				if(MFAC_Data.LogAutoTurnin) then print("   Quest Name:", quest_name, "  Can Handle:", can_handle); end;
				if(can_handle) then
					local item_data = MuffinFactionizer.QuestData[quest_name].items;
					local can_turn_it_in = true;
					for item, item_count in pairs(item_data) do
						local have = GetItemCount(item, true);
						local have_enough = (item_count <= have);
						if(MFAC_Data.LogAutoTurnin) then print("      For this I need ", item_count, item, "and I have ", have,"which is enough?", have_enough); end;
						can_turn_it_in = can_turn_it_in and have_enough;
					end
					if(can_turn_it_in) then
						MFAC.TurnInActive = true;
						SelectGossipAvailableQuest(i);
						break;
					end
				end
			end
		end
	elseif (event == "QUEST_PROGRESS") then
		if(MFAC_Data.LogAutoTurnin) then
			print("QUEST_PROGRESS")
			print("MFAC.TurnInActive", MFAC.TurnInActive);
			print("Quest is completable?", IsQuestCompletable())
		end
		if (MFAC.TurnInActive and IsQuestCompletable()) then
			CompleteQuest();
		end
	elseif (event == "QUEST_COMPLETE") then
		if(MFAC_Data.LogAutoTurnin) then
			print("QUEST_COMPLETE")
			print("MFAC.TurnInActive", MFAC.TurnInActive);
		end
		if (MFAC.TurnInActive) then
			GetQuestReward();
		end
	elseif (event == "QUEST_FINISHED") then
		if(MFAC_Data.LogAutoTurnin) then print("Quest finished"); end;
		MFAC.TurnInActive = false;
	elseif(event == "AZERITE_ITEM_EXPERIENCE_CHANGED" and MFAC_Data.WriteChatMessage) then
		local azeriteItemLocation = C_AzeriteItem.FindActiveAzeriteItem()
		if (azeriteItemLocation) then
			local azeriteItem = Item:CreateFromItemLocation(azeriteItemLocation);
			local xp, max_xp = C_AzeriteItem.GetAzeriteItemXPInfo(azeriteItemLocation)
			local powerLevel = C_AzeriteItem.GetPowerLevel(azeriteItemLocation)
			local txt = azeriteItem:GetItemLink() .. " " .. string.format(AZERITE_POWER_TOOLTIP_TITLE, powerLevel, (max_xp - xp))-- AZERITE_POWER_TOOLTIP_TITLE is "Level %s (%s Artifact Power to next level)";
			MFAC.Print(txt)
		end
	elseif (event == "PLAYER_REGEN_ENABLED" and MuffinFactionizer.set_watch_idx) then
		MFAC.update_faction_watch()
	end

	if (event == "QUEST_WATCH_UPDATE" and MFAC_Data.ShowToasts) then
		local _title, _, _, _, _, _, _, quest_id = GetQuestLogTitle(arg1);
			local faction_id = MuffinFactionizer.emissary_reward_quests[quest_id];
			if (faction_id) then
				MFAC.emissary_quest_checks = MFAC.emissary_quest_checks or {}
				MFAC.emissary_quest_checks[quest_id] = true
			end
	end
	if( event == "QUEST_LOG_UPDATE" and MFAC_Data.ShowToasts) then
		check_emissary_completion()
	end

	local debug_duration = debugprofilestop() - debug_start
	if (MFAC_Data.debug_profile) then print("Onevent:", event, format("%.2fms",debug_duration)); end
end

function MFAC.update_faction_watch()
	if(not MuffinFactionizer.set_watch_idx) then return end;

	if(InCombatLockdown()) then
		C_Timer.After(MuffinFactionizer.FactionUpdateRetryDelay, function()  MFAC.update_faction_watch();  end)
		return;
	end

	SetWatchedFactionIndex(MuffinFactionizer.set_watch_idx)
	MFAC.StatusTrackingBarManager_UpdateBarsShown();
	if (not MFAC_Data.SilentSwitch) then
		local name = GetFactionInfo(MuffinFactionizer.set_watch_idx)
		MFAC.Print(MFAC.Colours.Help .. ADDON_NAME .. ":|r " .. FIZ_TXT.switchBar .. " [" .. tostring(name) .. "|r]")
	end
	MuffinFactionizer.set_watch_idx = nil
end

-------------------------------
-- _04_ Addon Initialization --
-------------------------------
function FIZ_InitVariable(var, value)
	local result = 0
	if var == nil then return result end

	if (not MFAC_Data[var]) and (not MFAC_Data[var.."_inited"]) then
		-- this option is not Set, and has never been Set by default, do so and let the user know
		MFAC_Data[var] = value
		MFAC_Data[var.."_inited"] = true
		result = 1
	elseif (not MFAC_Data[var.."_inited"]) then
		-- the option is enabled, but not marked as inited, do so silently
		MFAC_Data[var.."_inited"] = true
	end

	return result
end

-------------------------------
function FIZ_Init()

	--
	-- Initialize all session data
	--
	local race_id = select(3, UnitRace("player"))
	local faction_name = select(2, UnitFactionGroup("player")) -- Localized faction name
	local class_id = select(3, UnitClass("player"))
	MFAC.player_name = UnitName("player")
	MFAC.realm_name = GetRealmName()

	if (IsInGuild()) then
		MFAC.GuildName = GetGuildInfo("player")
	end

	MFAC.IsHuman = (race_id == MFAC.RaceID.Human);
	MFAC.IsDeathKnight = (class_id == MFAC.ClassID.DeathKnight)
	MFAC.IsAlliance = (faction_name == FACTION_ALLIANCE)
	MFAC.IsHorde = (faction_name == FACTION_HORDE)

	--
	-- Initialize all persistent data
	--
	MFAC_Data = MFAC_Data or {}
	MFAC_Data.Bank = MFAC_Data.Bank or {}
	MFAC_Data.player_bank_key = MFAC.realm_name .. " - " .. MFAC.player_name
	MFAC_Data.Bank[MFAC_Data.player_bank_key] = MFAC_Data.Bank[MFAC_Data.player_bank_key] or {}

	--MFAC.RepHeaderSortByOptions = {
	--	None = 1,
	--	Reverse = 2,
	--};

	if(MFAC_Data.RepGroupBy == nil) then
		if (MFAC_Data.SortByStanding) then
			MFAC_Data.RepGroupBy = MFAC.RepGroupByOptions.Standing;
			MFAC_Data.SortByStanding = nil
		else
			MFAC_Data.RepGroupBy = MFAC.RepGroupByOptions.Expansion;
		end
	end

	--deprecated
	MFAC_Data.needsTip = nil;
	MFAC_Data.OriginalCollapsed = nil

	MFAC.CollapsedHeader = {}
	MFAC.build_frames()
	MFAC.preload_rep_tokens();
	MFAC.preload_reward_quests()

	local changed = 0
	for option, default in pairs(MFAC.all_option_defaults) do
		changed = changed + FIZ_InitVariable(option, default);
	end

	if (not FACTION_BAR_COLORS[9]) then
		FACTION_BAR_COLORS[9] = ARTIFACT_GOLD_COLOR; --luacheck: ignore 122
	end

	MFAC.ExtractSkills()

	-- Initialize Faction information
	MFAC.InitEnFactions()

	-- Faction-appropriate data. Neutrals only get the Neutral data.
	if (MFAC.IsAlliance) then
		MFAC.InitEnFactionGainsAlliance()
	elseif(MFAC.IsHorde) then
		MFAC.InitEnFactionGainsHorde()
	end

	FIZ_InitEnFactionGainsNeutral()

	MFAC.InitRepStore();

end

------------------------
-- _05_ Slash Handler --
------------------------

local setting_commands = {["enable"] = 1, ["disable"] = 1, ["toggle"] = 1};

local function set_option_var(p_var, p_action)

	if(p_action == "toggle") then
		MFAC_Data[p_var] = not MFAC_Data[p_var];
	elseif(p_action == "enable") then
		MFAC_Data[p_var] = true;
	elseif(p_action == "disable") then
		MFAC_Data[p_var] = false;
	end

end
function FIZ_SlashHandler(msg)
	local show_help = false;
	local made_changes = false;
	local msgLower = string.lower(msg)
	local wordsLower = FIZ_GetWords(msgLower)
	local command = wordsLower[0];
	local setting = wordsLower[1];
	local garbage = wordsLower[2]; -- Nothing takes this many arguments

	if (command == nil or garbage) then
		show_help = true;
	elseif (setting_commands[command] and setting and (MFAC.all_option_command_map[setting] or setting == "all")) then -- it's one of enable/disable/toggle and a valid setting was passed

		if (setting == "all") then
			for _k, v in pairs(MFAC.all_option_command_map) do
				set_option_var(v, command)
			end
		else
			local option = MFAC.all_option_command_map[setting];
			set_option_var(option, command)
		end

		made_changes = true;

	elseif (command == "list") then
		if (setting) then
			if (setting == "1" or setting == string.lower(_G["FACTION_STANDING_LABEL1"])) then
				MFAC.PrintByStanding(1)
			elseif (setting == "2" or setting == string.lower(_G["FACTION_STANDING_LABEL2"])) then
				MFAC.PrintByStanding(2)
			elseif (setting == "3" or setting == string.lower(_G["FACTION_STANDING_LABEL3"])) then
				MFAC.PrintByStanding(3)
			elseif (setting == "4" or setting == string.lower(_G["FACTION_STANDING_LABEL4"])) then
				MFAC.PrintByStanding(4)
			elseif (setting == "5" or setting == string.lower(_G["FACTION_STANDING_LABEL5"])) then
				MFAC.PrintByStanding(5)
			elseif (setting == "6" or setting == string.lower(_G["FACTION_STANDING_LABEL6"])) then
				MFAC.PrintByStanding(6)
			elseif (setting == "7" or setting == string.lower(_G["FACTION_STANDING_LABEL7"])) then
				MFAC.PrintByStanding(7)
			elseif (setting == "8" or setting == string.lower(_G["FACTION_STANDING_LABEL8"])) then
				MFAC.PrintByStanding(8)
			else
				show_help = true;
			end
		else
			MFAC.PrintByStanding()
		end
	elseif (command == "test") then
		MFAC.Test();
	elseif (command == "status") then
		FIZ_Status()
	elseif (command == "help") then --TODO: list all options
		FIZ_Help()
	elseif (command == "about") then
		FIZ_About()
	else
		show_help = true;
	end

	if	(show_help) then
		FIZ_PrintSlash(FIZ_TXT.command, msgLower)
	end

	if (made_changes) then
		if ( ReputationFrame:IsVisible() ) then
			ReputationFrame_Update();
		end
		if ( UI.ReputationDetailFrame:IsVisible()) then
			FIZ_BuildUpdateList()
			MFAC.UpdateDetailList()
		end
	end

end

-----------------------------------
-- _06_ General Helper Functions --
-----------------------------------


------------------------------------------------------------
function FIZ_Printtest(msg,msg1,msg2) --fpt
	MFAC.Print("" .. tostring(msg) .. " " .. tostring(msg1) .. "  " .. tostring(msg2), nil)
end
------------------------------------------------------------
function FIZ_PrintSlash(msg, msg1)
	MFAC.Print(MFAC.Colours.Help .. ADDON_NAME .. ":|r " .. msg .. " [" .. MFAC.Colours.Help .. msg1 .. "|r]", true)
	FIZ_Help()
end

------------------------------------------------------------
function FIZ_TableSize(info) --TODO: replace all calls to this with calls to MFAC.table_size
	local result = 0
	if info then
		for _item in pairs(info) do result = result + 1 end
	end
	return result
end

------------------------------------------------------------
function FIZ_GetWords(str)
	local ret = {};
	local pos=0;
	local index=0
	while(true) do
		local word, _;
		_, pos, word = string.find(str, "^ *([^%s]+) *", pos+1);
		if(not word) then
			return ret;
		end
		ret[index]=word
		index = index+1
	end
end

------------------------------------------------------------
function FIZ_Concat(list, start, stop)
	local ret = "";

	if (start == nil) then start = 0 end
	if (stop == nil) then stop = FIZ_TableSize(list) end

	for i = start,stop do
		if list[i] then
			if (ret ~= "") then ret = ret.." " end
			ret = ret..list[i]
		end
	end
	return ret
end

------------------------------------------------------------


------------------------
-- _07_ information
------------------------

local bool_options_string = nil;

function FIZ_Help()

	if(not bool_options_string) then
		bool_options_string = ""
		for k in pairs(MFAC.all_option_command_map) do
			bool_options_string = k .. " | " .. bool_options_string;
		end
		bool_options_string = bool_options_string .. "all";
	end

	MFAC.Print(" ", true)
	MFAC.Print(MFAC.Colours.Help .. ADDON_NAME .. ":|r " .. FIZ_TXT.help, true)
	MFAC.Print(MFAC.Colours.Help .. FIZ_TXT.usage .. ":|r /mfac help " .. MFAC.Colours.Help .. FIZ_TXT.helphelp, true)
	MFAC.Print(MFAC.Colours.Help .. FIZ_TXT.usage .. ":|r /mfac about " .. MFAC.Colours.Help .. FIZ_TXT.helpabout, true)
	MFAC.Print(MFAC.Colours.Help .. FIZ_TXT.usage .. ":|r /mfac status " .. MFAC.Colours.Help .. FIZ_TXT.helpstatus, true)
	MFAC.Print(MFAC.Colours.Help .. FIZ_TXT.usage .. ":|r /mfac list " .. MFAC.Colours.Help .. FIZ_TXT.helplist, true)
	MFAC.Print(MFAC.Colours.Help .. FIZ_TXT.usage .. ":|r /mfac enable <setting>", true)
	MFAC.Print(MFAC.Colours.Help .. FIZ_TXT.usage .. ":|r /mfac disable <setting>", true)
	MFAC.Print(MFAC.Colours.Help .. FIZ_TXT.usage .. ":|r /mfac toggle <setting>", true)
	MFAC.Print(MFAC.Colours.Help .. "  <setting>:|r" .. bool_options_string, true)
end
------------------------------------------------------------
function FIZ_About()
	local ver = GetAddOnMetadata("MuffinFactionizer", "Version");
	local date = GetAddOnMetadata("MuffinFactionizer", "X-Date");
	local author = GetAddOnMetadata("MuffinFactionizer", "Author");
	local web = GetAddOnMetadata("MuffinFactionizer", "X-Website");

	if (author ~= nil) then
		MFAC.Print(ADDON_NAME .." "..FIZ_TXT.by.." "..MFAC.Colours.Help..author.."|r", true);
	end
	if (ver ~= nil) then
		MFAC.Print("  "..FIZ_TXT.version..": "..MFAC.Colours.Help..ver.."|r", true);
	end
	if (date ~= nil) then
		MFAC.Print("  "..FIZ_TXT.date..": "..MFAC.Colours.Help..date.."|r", true);
	end
	if (web ~= nil) then
		MFAC.Print("  "..FIZ_TXT.web..": "..MFAC.Colours.Help..web.."|r", true);
	end

end
------------------------------------------------------------
function FIZ_Status()
	MFAC.Print("|r" .. MFAC.Colours.Help .. ADDON_NAME .. ":|r " .. FIZ_TXT.status, true)

	for k in pairs(MFAC.all_option_defaults) do
		MFAC.Print("   " .. MFAC.TXT["status" .. k] .. ": " .. MFAC.Colours.Help .. MFAC.BoolToEnabled(MFAC_Data[k]) .. "|r", true)
	end

end

-----------------------------------
-- _08_ Faction map --
-----------------------------------
function MFAC.InitMapName(p_map_id)

	local map_name;

	if p_map_id == MuffinFactionizer.zone.Spillover_Rep_from_Dungeons then
		map_name = FIZ_TXT.srfd
	elseif p_map_id == MuffinFactionizer.zone.Normal_Cataclysm_Instances then
		map_name = FIZ_TXT.nci
	elseif p_map_id == MuffinFactionizer.zone.Heroic_Cataclysm_Instances then
		map_name = FIZ_TXT.hci
	elseif not p_map_id then
		local stack_trace = debugstack(2, 1, 1);
		print("Missing MapID")
		print("      ", string.match(stack_trace, "(%S+.lua:%d+)"));
		map_name = " "
	end

	if not map_name then
		local map_info = C_Map.GetMapInfo(p_map_id);
		if(map_info) then
			map_name = map_info.name;
		end
	end

	if not map_name then
		map_name = C_Map.GetAreaInfo(p_map_id)
	end

	if not map_name then
		local stack_trace = debugstack(2, 1, 1);
		print("Bad MapID:", p_map_id)
		print("      ", string.match(stack_trace, "(%S+.lua:%d+)"));
		map_name = p_map_id
	end

	MFAC.DebugPrint("MFAC.InitMapName", p_map_id, map_name)

	return map_name
end

function FIZ_Initmobname(fimob)
	--- fpt f_ion	FIZ_Printtest(fimob,"","mob 1")
	local mob
	if fimob== 2 then
		mob = FIZ_TXT.oboss
	elseif fimob== 3 then
		mob = FIZ_TXT.aboss
	elseif fimob == 4 then
		mob = FIZ_TXT.pboss
	elseif fimob == 5 then
		mob = FIZ_TXT.fclear
	elseif fimob == 11 then
		mob = (FIZ_TXT.AU.." "..FIZ_TXT.BB)
	elseif fimob== 12 then
		mob = (FIZ_TXT.AU.." "..FIZ_TXT.SSP)
	elseif fimob== 13 then
		mob = (FIZ_TXT.AU.." "..FIZ_TXT.Wa)
	elseif fimob == 14 then
		mob = FIZ_TXT.VCm
	elseif fimob == 15 then
		mob = (FIZ_TXT.AN.." "..FIZ_TXT.BB)
	elseif fimob== 16 then
		mob = (FIZ_TXT.AN.." "..FIZ_TXT.SSP)
	elseif fimob== 17 then
		mob = (FIZ_TXT.AN.." "..FIZ_TXT.Wa)
	else
	--[[--	local mobName = GetmobNameByID(fimob);
		mob = mobName	--]]--
	end
	--- fpt f_ion	FIZ_Printtest(fimob,mob,"mob 2")
	if not mob then
		mob = fimob
	end
	mark_mob = mob
end

function MFAC.GetItemNameForID(p_item)
	local item_name

	if p_item==1 then
		item_name = FIZ_TXT.cdq
	elseif p_item==2 then
		item_name = FIZ_TXT.fdq
	elseif p_item==3 then
		item_name = FIZ_TXT.ndq
	elseif p_item == 4 then
		item_name = FIZ_TXT.cbadge
--		local stack_trace = debugstack(3, 1, 1);
--		print("      ", string.match(stack_trace, "(%S+.lua:%d+)"));
	elseif(type(p_item) ~= "number") then
		local stack_trace = debugstack(3, 1, 1);
		print("      ", string.match(stack_trace, "(%S+.lua:%d+)"));
	else
		if(not MFAC_Data.ItemNameCache) then
			MFAC_Data.ItemNameCache = {};
		end
		item_name = GetItemInfo(p_item) or MFAC_Data.ItemNameCache[p_item]
		if(item_name) then
			return item_name;
		end

		if (type(p_item) == "number") then
			MFAC.CacheMiss[p_item] = 1;
			MFAC.MainWindow:RegisterEvent("GET_ITEM_INFO_RECEIVED")
		elseif (type(p_item) == "string") then
			local item_id = GetItemInfoInstant(p_item);
			if (item_id) then
				MFAC.CacheMiss[item_id] = 1;
				MFAC.MainWindow:RegisterEvent("GET_ITEM_INFO_RECEIVED")
			end
		end
	end

	if not item_name then
		item_name = p_item
	end

	return item_name;
end

--local MFACScanningTooltip = CreateFrame("GameTooltip", "MFACScanningTooltip", UIParent, "GameTooltipTemplate")
--
--local quest_names = setmetatable({}, {
--	__index = function(t, id_num)
--		MFACScanningTooltip:SetOwner(UIParent, "ANCHOR_NONE")
--		MFACScanningTooltip:SetHyperlink("quest:" .. id_num)
--		local quest_name = MFACScanningTooltipTextLeft1:GetText()
--		MFACScanningTooltip:Hide()
--
--		--TODO: Replace this with something more reasonable. Several of these are also valid quest IDs, so this makes no sense.
--		if id_num == 1 then
--			quest_name = FIZ_TXT.cdq
--		elseif id_num == 2 then
--			quest_name = FIZ_TXT.coq
--		elseif id_num == 3 then
--			quest_name = FIZ_TXT.fdq
--		elseif id_num == 4 then
--			quest_name = FIZ_TXT.foq
--		elseif id_num == 5 then
--			quest_name = FIZ_TXT.ndq
--		elseif id_num == 6 then
--			quest_name = FIZ_TXT.deleted
--		elseif id_num == 7 then
--			quest_name = FIZ_TXT.Championing
--		elseif id_num == 8 then
--			quest_name = FIZ_TXT.bpqfg
--		elseif id_num== 99 then
--			quest_name = FIZ_TXT.tbd
--		else
--			if not quest_name then
--				return id_num
--			end
--			t[id_num] = quest_name
--		end
--		return quest_name
--	end
--})

--TODO: Is this even used??
function FIZ_GetTabardFaction()
	for i = 1, 40 do
		local _, _, _, _, _, _, _, _, _, _, id = UnitBuff("player", i)
		if not id then
			break
		end
		local data = championFactions[id]
		if data then
			local faction, level = data[2], data[1]
			MFAC.DebugPrint("GetChampionedFaction:", faction, level);
			return faction, level
		end
	end
	MFAC.DebugPrint("GetChampionedFaction:", "none");
end

MFAC.factor_cache = {};

function MFAC.GetRepScaleForFaction(p_faction_id)

	if(MFAC.factor_cache[p_faction_id]) then
		return MFAC.factor_cache[p_faction_id];
	end

	local factor = 1.0

	if MFAC.IsHuman then
		factor = factor + 0.1
	end

	local hasBonusRepGain = select(15, GetFactionInfoByID(p_faction_id))
	if (hasBonusRepGain) then
		factor = factor + 1
	end

	MFAC.factor_cache[p_faction_id] = factor;

	return factor;

end

------------------------------------
-- _09_ Faction Lists --
------------------------------------

function FIZ_AddMob(p_faction_id, p_from, p_to, p_name, p_rep_amount, p_zone, p_limit)

	local stack_trace = debugstack(2, 1, 1);

	if(p_faction_id == nil or (type(p_faction_id) ~= "number")) then
		print("      Invalid Faction ID:", p_faction_id, string.match(stack_trace, "(%S+.lua:%d+)"));
	end

	MFAC.ValidateFromTo(p_from, p_to, stack_trace);

	if(p_name == nil or (type(p_name) ~= "string")) then
		print("      Invalid Mob Name:", p_name, string.match(stack_trace, "(%S+.lua:%d+)"));
	end

	if((p_rep_amount == nil) or (type(p_rep_amount) ~= "number")) then
		print("      Invalid rep:", p_rep_amount, string.match(stack_trace, "(%S+.lua:%d+)"));
	end

	if((p_zone == nil) or (type(p_zone) ~= "number")) then
		print("      Invalid Zone:", p_zone, string.match(stack_trace, "(%S+.lua:%d+)"));
	end


--	local standing = MuffinFactionizer.standing;

--	if(
--		not (p_faction_id and p_from and p_to and p_name and p_rep_amount) or	--All of these need to be set
--		(type(p_rep_amount) ~= "number") or  --reputation needs to be a number
--		(type(p_faction_id) ~= "number") or  -- faction needs to be a number
--		(p_name == 2) or
--	) then
--		print("FIZ_AddMob error:","faction:", p_faction, "from:", p_from, "to:", p_to, "name:", p_name, "Rep:", p_rep_amount);
--		local stack_trace = debugstack(2, 1, 1);
--		print("      ", string.match(stack_trace, "(%S+.lua:%d+)"));
--		return;
--	end

	--TODO: Validate that p_name is either a string or a valid number (or make it so it's always a string)


	FIZ_Initmobname(p_name)
	local map_name = MFAC.InitMapName(p_zone)
	local rep_amount = p_rep_amount * MFAC.GetRepScaleForFaction(p_faction_id)


	local faction_info = FIZ_FactionGain[p_faction_id]
	if not faction_info then
		faction_info = {}
		FIZ_FactionGain[p_faction_id] = faction_info
	end

	local new_mob = {}
	new_mob.name = p_name
	new_mob.rep = rep_amount
	new_mob.zone = map_name
	new_mob.maxStanding = p_to

	local standing = p_from
	repeat
		faction_info[standing] = faction_info[standing] or {}
		local standing_info = faction_info[standing];

		local mob_list = standing_info.mobs
		if mob_list then
			mob_list.count = mob_list.count + 1
		else
			mob_list = {}
			mob_list.data = {}
			mob_list.count = 0
			standing_info.mobs = mob_list
		end

		mob_list.data[mob_list.count] = new_mob

		standing = standing + 1

		MFAC.DebugPrint("Added mob [", p_name, "] for faction [", p_faction_id, "] and standing [", _G["FACTION_STANDING_LABEL" .. standing], "]")
	until standing >= p_to
end

function MFAC.AddQuest(p_faction_id, p_from, p_to, p_quest_id, rep, p_item_list, p_required_profession)

	local stack_trace = debugstack(3, 1, 1);

	if(p_faction_id == nil or (type(p_faction_id) ~= "number")) then
		print("      Invalid Faction ID:", p_faction_id, string.match(stack_trace, "(%S+.lua:%d+)"));
	end

	MFAC.ValidateFromTo(p_from, p_to, stack_trace);

	if(p_quest_id == nil or (type(p_quest_id) ~= "number")) then
		print("      Invalid Quest ID:", p_quest_id, string.match(stack_trace, "(%S+.lua:%d+)"));
	end

	if((rep == nil) or (type(rep) ~= "number")) then
		print("      Invalid rep:", rep, string.match(stack_trace, "(%S+.lua:%d+)"));
	end

	local items_okay = (type(p_item_list) == "table") or (p_item_list == nil);
	if(not items_okay) then
		print("      Invalid item list:", p_item_list, string.match(stack_trace, "(%S+.lua:%d+)"));
	end

	if(not(type(p_required_profession) == "number" or p_required_profession == nil)) then
		print("      Invalid profession:", p_required_profession, string.match(stack_trace, "(%S+.lua:%d+)"));
	end

	rep = rep * MFAC.GetRepScaleForFaction(p_faction_id)

	local faction_info = FIZ_FactionGain[p_faction_id]
	if not faction_info then
		faction_info = {}
		FIZ_FactionGain[p_faction_id] = faction_info
	end

	local new_quest = {}
	new_quest.name = MFAC.GetQuestNameForID(p_quest_id)
	new_quest.id = p_quest_id
	new_quest.rep = rep
	new_quest.maxStanding = p_to

	if(p_required_profession) then
		new_quest.profession = p_required_profession;
	end

	if (p_item_list) then
		new_quest.items = {}
		for item, num_items in pairs(p_item_list) do
			local item_name = MFAC.GetItemNameForID(item)
			new_quest.items[item_name] = num_items;
		end

		MuffinFactionizer.QuestData[new_quest.name] = new_quest;

	end


	for standing = p_from, p_to do
		faction_info[standing] = faction_info[standing] or {}
		local standing_info = faction_info[standing]

		local quest_list = standing_info.quest_list
		if quest_list then
			quest_list.count = quest_list.count + 1
		else
			quest_list = {}
			quest_list.data = {}
			quest_list.count = 0
			standing_info.quest_list = quest_list
		end

		quest_list.data[quest_list.count] = new_quest

		--print("Added quest [" .. p_quest_id .. "] for faction [" .. p_faction_id .. "] and standing [" .. _G["FACTION_STANDING_LABEL" .. standing] .. "]");
	end
end

function MFAC.AddTurnInNPC(p_faction_id, p_npc_id, p_quest_ids)

	local stack_trace = debugstack(3, 1, 1);

	if(p_faction_id == nil or (type(p_faction_id) ~= "number")) then
		print("      Invalid Faction ID:", p_faction_id, string.match(stack_trace, "(%S+.lua:%d+)"));
	end

	if(p_npc_id == nil or (type(p_npc_id) ~= "number")) then
		print("      Invalid NPC ID:", p_faction_id, string.match(stack_trace, "(%S+.lua:%d+)"));
	end

	if(p_quest_ids == nil or (type(p_quest_ids) ~= "table") or (next(p_quest_ids) == nil)) then
		print("      Invalid Quest IDs:", p_faction_id, string.match(stack_trace, "(%S+.lua:%d+)"));
	end


	local npcs = MuffinFactionizer.TurnInNPC;

	--if (npcs[p_npc_id]) then
	--	print("      Adding already known NPC:", p_npc_id, string.match(stack_trace, "(%S+.lua:%d+)")); TODO: Fix this properly
	--end

	local info = {};
	info.faction_id = p_faction_id;
	info.quest_ids = p_quest_ids;
	info.quest_names = {};

	for _k,v in pairs(p_quest_ids) do
		local name = MFAC.GetQuestNameForID(v);
		info.quest_names[name] = name;
	end

	npcs[p_npc_id] = info;
end

function FIZ_AddInstance(p_faction_id, p_from, p_to, name, rep, heroic)
	if not p_faction_id then return end
	if not name then return end
	if not rep then return end
	if (type(rep) ~= "number") then return end

	local stack_trace = debugstack(3, 1, 1);

	MFAC.ValidateFromTo(p_from, p_to, stack_trace);

	local map_name = MFAC.InitMapName(name)

	rep = rep * MFAC.GetRepScaleForFaction(p_faction_id)

	for standing = p_from,p_to do
		local faction_info = FIZ_FactionGain[p_faction_id]
		if not faction_info then
			faction_info = {}
			FIZ_FactionGain[p_faction_id] = faction_info
		end
		local standing_info = faction_info[standing]
		if not standing_info then
			standing_info = {}
			faction_info[standing] = standing_info
		end
		local add_info = standing_info.instance
		if add_info then
			add_info.count = add_info.count + 1
		else
			add_info = {}
			add_info.data = {}
			add_info.count = 0
			standing_info.instance = add_info
		end
		local count = add_info.count
		add_info.data[count] = {}
		local add_count=add_info.data[count]
		add_count.name = map_name
		add_count.rep = rep
		add_count.maxStanding = p_to
		if (heroic) then
			add_count.level = FIZ_TXT.heroic
		else
			add_count.level = FIZ_TXT.normal
		end


		MFAC.DebugPrint("Added instance [", name, "] for faction [", p_faction_id, "] and standing [", _G["FACTION_STANDING_LABEL" .. standing], "]")
	end
end


function MFAC.AddItems(p_faction_id, p_from, p_to, p_rep_amount, p_item_list)

	local stack_trace = debugstack(3, 1, 1);

	if(p_faction_id == nil or (type(p_faction_id) ~= "number")) then
		print("      Invalid Faction ID:", p_faction_id, string.match(stack_trace, "(%S+.lua:%d+)"));
	end

	MFAC.ValidateFromTo(p_from, p_to, stack_trace);

	if((p_rep_amount == nil) or (type(p_rep_amount) ~= "number")) then
		print("      Invalid rep:", p_rep_amount, string.match(stack_trace, "(%S+.lua:%d+)"));
	end

	if(p_item_list == nil or type(p_item_list) ~= "table") then
		print("      Invalid item list:", tostring(p_item_list), string.match(stack_trace, "(%S+.lua:%d+)"));
	end


	p_rep_amount = p_rep_amount * MFAC.GetRepScaleForFaction(p_faction_id)

	local faction_info = FIZ_FactionGain[p_faction_id]
	if not faction_info then
		faction_info = {}
		FIZ_FactionGain[p_faction_id] = faction_info
	end

	local group_name = nil
	local group_count = nil
	for item_id, item_count in pairs(p_item_list) do
		group_name = group_name or MFAC.GetItemNameForID(item_id)
		group_count = group_count or item_count
		if(not MuffinFactionizer.ItemToFactionMap[item_id]) then
			MuffinFactionizer.ItemToFactionMap[item_id] = {};
		end
		if(not MuffinFactionizer.ItemToFactionMap[item_id][p_faction_id]) then
			MuffinFactionizer.ItemToFactionMap[item_id][p_faction_id] = 1;
		end
	end

	local standing = p_from
	repeat
		faction_info[standing] = faction_info[standing] or {}
		local standing_info = faction_info[standing];

		standing_info.items = standing_info.items or {}
		local item_list = standing_info.items

		local new_item = {}
		new_item.list = p_item_list
		new_item.count = group_count
		new_item.name = group_name
		new_item.rep = p_rep_amount
		new_item.maxStanding = p_to

		tinsert(item_list, new_item)

		MFAC.DebugPrint("AddItem: Added item [", new_item.name, "] for faction [", p_faction_id, "] and standing [", _G["FACTION_STANDING_LABEL" .. standing], "]")

		standing = standing + 1

	until standing >= p_to

end

--These are used for tooltips when hovering over currencies. Notably the Mission Rewards in BfA
function MFAC.AddCurrency(p_faction_id, p_currency_id)
	local stack_trace = debugstack(3, 1, 1);

	if(p_faction_id == nil or (type(p_faction_id) ~= "number")) then
		print("      Invalid Faction ID:", p_faction_id, string.match(stack_trace, "(%S+.lua:%d+)"));
	end

	if(p_currency_id == nil or (type(p_currency_id) ~= "number")) then
		print("      Invalid Currency ID:", p_currency_id, string.match(stack_trace, "(%S+.lua:%d+)"));
	end

	if(not MuffinFactionizer.CurrencyToFactionMap[p_currency_id]) then
		MuffinFactionizer.CurrencyToFactionMap[p_currency_id] = {};
	end
	if(not MuffinFactionizer.CurrencyToFactionMap[p_currency_id][p_faction_id]) then
		MuffinFactionizer.CurrencyToFactionMap[p_currency_id][p_faction_id] = 1;
	end


end



function FIZ_AddGeneral(p_faction_id, p_from, p_to, name, rep, head, tip, tipList, flag)
	local stack_trace = debugstack(3, 1, 1);

	if not p_faction_id then return end
	if not rep then return end
	if not name then return end
	if (type(rep) ~= "number") then return end

	MFAC.ValidateFromTo(p_from, p_to, stack_trace);

	--TODO: Remove this abomination
	if name == "1" then
		name = FIZ_TXT.tfr
		head = FIZ_TXT.tfr
		tip = FIZ_TXT.nswts
	end

	rep = rep * MFAC.GetRepScaleForFaction(p_faction_id)

	local tipString = ""
	for standing = p_from,p_to do
		local faction_info = FIZ_FactionGain[p_faction_id]
		if not faction_info then
			faction_info = {}
			FIZ_FactionGain[p_faction_id] = faction_info
		end
		local standing_info = faction_info[standing]
		if not standing_info then
			standing_info = {}
			faction_info[standing] = standing_info
		end
		local add_info = standing_info.general
		if add_info then
			add_info.count = add_info.count + 1
		else
			add_info = {}
			add_info.data = {}
			add_info.count = 0
			standing_info.general = add_info
		end
		local count = add_info.count
		add_info.data[count] = {}
		local add_count=add_info.data[count]
		add_count.name = name
		add_count.flag = flag
		add_count.head = head
		add_count.tip = tip
		add_count.rep = rep
		add_count.maxStanding = p_to
		if (tipList) then
			add_count.tipList = {}
			for tip in pairs(tipList) do
				if tipString ~= "" then tipString = tipString..", " end
				tipString = tipString..tipList[tip]..": "..tip
				add_count.tipList[tip] = tipList[tip]
			end
		end

		MFAC.DebugPrint("AddGeneral: Added general rep gain [", name, "] for faction [", p_faction_id, "] and standing [", _G["FACTION_STANDING_LABEL"..standing], "] with tooltip [", tipString, "]")
	end
end

function MFAC.normalize_rep_values(p_min, p_max, p_current)

	-- Normalize Values to within standing
	local normalized_max = p_max - p_min
	local normalized_current = p_current - p_min

	return 0, normalized_max, normalized_current
end

-----------------------------------
-- _10_ New Hook Functions --
-----------------------------------



function MFAC.ExpandFactionHeader(p_index)

	if  MFAC_Data.RepGroupBy == MFAC.RepGroupByOptions.Standing then
		MFAC.CollapsedHeader[p_index] = false
		MFAC.ReputationFrame_Update()
	else
		MFAC.Orig_ExpandFactionHeader(p_index)
	end
end

function MFAC.CollapseFactionHeader(p_index)

	if  MFAC_Data.RepGroupBy == MFAC.RepGroupByOptions.Standing then
		MFAC.CollapsedHeader[p_index] = true
		MFAC.ReputationFrame_Update()
	else
		MFAC.Orig_CollapseFactionHeader(p_index)
	end

end

function FIZ_ReputationFrame_SetRowType(factionRow, isChild, isHeader, hasRep) 	--rowType is a binary table of type isHeader, isChild
	local factionRowName = factionRow:GetName()
	local factionBar = _G[factionRowName.."ReputationBar"];
	local factionTitle = _G[factionRowName.."FactionName"];
	local factionButton = _G[factionRowName.."ExpandOrCollapseButton"];
	local factionStanding = _G[factionRowName.."ReputationBarFactionStanding"];
	local factionBackground = _G[factionRowName.."Background"];
	local factionLeftTexture = _G[factionRowName.."ReputationBarLeftTexture"];
	local factionRightTexture = _G[factionRowName.."ReputationBarRightTexture"];

	factionLeftTexture:SetWidth(62);
	factionRightTexture:SetWidth(42);
	factionBar:SetPoint("RIGHT", factionRow, "RIGHT", 0, 0);
	if ( isHeader ) then
		if (isChild) then
			factionRow:SetPoint("LEFT", ReputationFrame, "LEFT", 29, 0);
		else
			factionRow:SetPoint("LEFT", ReputationFrame, "LEFT", 10, 0);
		end
		factionTitle:SetWidth(185);
		factionButton:SetPoint("LEFT", factionRow, "LEFT", 3, 0);
		factionButton:Show();
		factionTitle:SetPoint("LEFT",factionButton,"RIGHT",10,0);
		factionTitle:SetFontObject(GameFontNormalLeft);
		factionBackground:Hide()
		factionLeftTexture:SetHeight(15);
		factionLeftTexture:SetWidth(60);
		factionRightTexture:SetHeight(15);
		factionRightTexture:SetWidth(39);
		factionLeftTexture:SetTexCoord(0.765625, 1.0, 0.046875, 0.28125);
		factionRightTexture:SetTexCoord(0.0, 0.15234375, 0.390625, 0.625);
		factionBar:SetWidth(99);
	else
		if ( isChild ) then
			factionRow:SetPoint("LEFT", ReputationFrame, "LEFT", 52, 0);
		else
			factionRow:SetPoint("LEFT", ReputationFrame, "LEFT", 34, 0);
		end
		factionBar:SetWidth(101)
		factionTitle:SetWidth(160);
		factionButton:Hide();
		factionTitle:SetPoint("LEFT", factionRow, "LEFT", 10, 0);
		factionTitle:SetFontObject(GameFontHighlightSmall);
		factionBackground:Show();
		factionLeftTexture:SetHeight(21);
		factionRightTexture:SetHeight(21);
		factionLeftTexture:SetTexCoord(0.7578125, 1.0, 0.0, 0.328125);
		factionRightTexture:SetTexCoord(0.0, 0.1640625, 0.34375, 0.671875);

	end
	if ( (hasRep) or (not isHeader) ) then
		factionStanding:Show();
		factionBar:Show();
		factionBar:GetParent().hasRep = true;
	else
		factionStanding:Hide();
		factionBar:Hide();
		factionBar:GetParent().hasRep = false;
	end
end


MFAC.last_rep_bar_index = nil
function MFAC.ReputationBar_OnClick(self)
	UI.ReputationOptionsFrame:Hide();

	if(not MFAC_Data.ExtendDetails) then
		return;
	end

	local clicked_faction_id = self.faction_id

	if ((UI.ReputationDetailFrame:IsVisible()) and (clicked_faction_id == MFAC.last_rep_bar_index) ) then
		ReputationDetailFrame:Hide();
		UI.ReputationDetailFrame:Hide();
	elseif (self.hasRep) then
		MFAC.last_rep_bar_index = self.index;

		UI.ReputationDetailFrame.faction_id = clicked_faction_id
		UI.ReputationDetailFrame:Show()
		ReputationDetailFrame:Hide();
		UI.ReputationOptionsFrame:Hide();

		FIZ_BuildUpdateList();
		MFAC.UpdateDetailList(1)

		ReputationFrame_Update();
	end

end

function MFAC.get_colour_for_detail_entry(p_entry, p_default)
	local colour = p_default

	if (p_entry.highlight) then
		colour = MFAC.Colours.Highlight
	elseif (p_entry.suppress) then
		colour = MFAC.Colours.Suppress
	elseif (p_entry.lowlight) then
		colour = MFAC.Colours.Lowlight
	end

	return colour;
end

function MFAC.get_detail_texture_for_entry(p_entry)
	local texture_name
	local postfix = ""

	if (not p_entry.suppress) then
		postfix = "-Green"
	end
	if (p_entry.hasList) then
		if (p_entry.listShown) then
			texture_name = "Interface/Addons/" .. ADDON_NAME .. "/Textures/UI-MinusButton-Up" .. postfix .. ".tga"
		else
			texture_name = "Interface/Addons/" .. ADDON_NAME .. "/Textures/UI-PlusButton-Up" .. postfix .. ".tga"
		end
	else
		texture_name = "Interface/Addons/" .. ADDON_NAME .. "/Textures/UI-EmptyButton-Up" .. postfix .. ".tga"
	end

	return texture_name
end

function MFAC.detail_update_normal_entry_row(p_button, p_entry)
	local colour = MFAC.get_colour_for_detail_entry(p_entry, MFAC.Colours.NormalEntry)

	p_button.mfac_label:SetText(colour .. p_entry.type)
	p_button.mfac_name:SetText(colour .. p_entry.name)
	p_button.mfac_rep:SetText(colour .. p_entry.rep)
	p_button.mfac_times:SetText(colour .. p_entry.times)

	if (not p_entry.canSuppress) then
		p_button.mfac_texture:Hide()
	else
		p_button.mfac_texture:Show()
		p_button.mfac_texture:SetTexture(MFAC.get_detail_texture_for_entry(p_entry))
	end
end

function MFAC.detail_update_item_entry_row(p_button, p_entry)
	local colour = MFAC.get_colour_for_detail_entry(p_entry, MFAC.Colours.Suppress)

	p_button.mfac_texture:Hide()
	p_button.mfac_label:SetText("")
	p_button.mfac_name:SetText(colour .. p_entry.name)
	p_button.mfac_rep:SetText("")
	p_button.mfac_times:SetText(colour .. p_entry.times)
end

function MFAC.UpdateDetailList(p_offset)
	local UPDATE_LIST_HEIGHT = 13

	if (not UI.ReputationDetailFrame:IsVisible()) then return end

	UI.detail_show_quests:SetChecked(MFAC_Data.ShowQuests)
	UI.detail_show_instances:SetChecked(MFAC_Data.ShowInstances)
	UI.detail_show_mobs:SetChecked(MFAC_Data.ShowMobs)
	UI.detail_show_items:SetChecked(MFAC_Data.ShowItems)
	UI.detail_general:SetChecked(MFAC_Data.ShowGeneral)
	UI.detail_show_all:SetChecked(MFAC_Data.ShowQuests and MFAC_Data.ShowInstances and MFAC_Data.ShowMobs and MFAC_Data.ShowItems and MFAC_Data.ShowGeneral)

	local entryIndex = 1;

	local numEntries = FIZ_GetUpdateListSize()
--print("MFAC.UpdateDetailList", numEntries, UPDATE_LIST_HEIGHT, UI.detail_slider:GetValue(), p_offset)
	if(numEntries > UPDATE_LIST_HEIGHT) then
		if (p_offset) then UI.detail_slider:SetValue(p_offset); end;
		UI.detail_slider:SetWidth(UI.detail_slider.original_width);
		UI.detail_slider:GetThumbTexture():Show()
		UI.detail_slider:SetMinMaxValues(1, numEntries - UPDATE_LIST_HEIGHT + 1)

		UI.detail_slider_button_scroll_up:Show()
		UI.detail_slider_button_scroll_down:Show()
		entryIndex = UI.detail_slider:GetValue();
	else
		UI.detail_slider:SetWidth(1)
		UI.detail_slider:GetThumbTexture():Hide()
		UI.detail_slider_button_scroll_up:Hide()
		UI.detail_slider_button_scroll_down:Hide()
	end

	local have_info = false;
	for i = 1, UPDATE_LIST_HEIGHT do
		local b = UI.detail_entry_buttons[i]
		while(FIZ_UpdateList[entryIndex] and not FIZ_UpdateList[entryIndex].isShown) do
			entryIndex = entryIndex + 1
		end
		local entry = FIZ_UpdateList[entryIndex]
		if(not entry) then
			b:Hide()
		else
			have_info = true
			b:Show()
--print(i, entryIndex, entry.name, "has list:", entry.hasList)

			b.id = entry.index
			b.tooltipHead = entry.tooltipHead
			b.tooltipTip = entry.tooltipTip
			b.tooltipDetails = entry.tooltipDetails
			if(entry.type ~= "") then --normal entry
				MFAC.detail_update_normal_entry_row(b, entry)
			else --Item entry
				MFAC.detail_update_item_entry_row(b, entry)
			end
		end
		entryIndex = entryIndex + 1
	end

	if have_info then
		UI.detail_no_info_text:Hide()
	else
		UI.detail_no_info_text:Show()
	end
end


function FIZ_MouseButtonUp(self, button)
	if (button and button == "LeftButton") then
		FIZ_UpdateEntryClick(self)
	elseif (button and button == "RightButton") then
		FIZ_UpdateEntrySuppress(self)
	end
end

function FIZ_UpdateEntryClick(self)
	-- sub function of FIZ_MouseButtonUp
	if (FIZ_UpdateList[self.id] and FIZ_UpdateList[self.id].hasList) then
		if (FIZ_UpdateList[self.id].listShown) then
			FIZ_ShowUpdateEntry(self.id, false)
		else
			FIZ_ShowUpdateEntry(self.id, true)
		end
	end
end

function FIZ_UpdateEntrySuppress(self)

	if (FIZ_UpdateList[self.id]) then
		if (FIZ_UpdateList[self.id].type ~= "") then
			if (FIZ_UpdateList[self.id].faction and FIZ_UpdateList[self.id].originalName) then
				if (not MFAC_Suppressed) then
					MFAC_Suppressed = {}
				end
				if (not MFAC_Suppressed[FIZ_UpdateList[self.id].faction]) then
					MFAC_Suppressed[FIZ_UpdateList[self.id].faction] = {}
				end
				if (MFAC_Suppressed[FIZ_UpdateList[self.id].faction][FIZ_UpdateList[self.id].originalName]) then
					--MFAC.Print("No longer suppressing ["..FIZ_UpdateList[self.id].faction.."]["..FIZ_UpdateList[self.id].originalName.."]");
					MFAC_Suppressed[FIZ_UpdateList[self.id].faction][FIZ_UpdateList[self.id].originalName] = nil
				else
					--MFAC.Print("Suppressing ["..FIZ_UpdateList[self.id].faction.."]["..FIZ_UpdateList[self.id].originalName.."]");
					MFAC_Suppressed[FIZ_UpdateList[self.id].faction][FIZ_UpdateList[self.id].originalName] = true
				end
				FIZ_BuildUpdateList()
			end
		end
	end
end

function FIZ_SupressNone(allFactions)

	if (allFactions == true) then
		MFAC_Suppressed = {}
		FIZ_BuildUpdateList()
	else
		local factionIndex = GetSelectedFaction()
		local faction_id = select(14, GetFactionInfo(factionIndex))

		if (faction_id) then
			if (not MFAC_Suppressed) then
				MFAC_Suppressed = {}
			end
			MFAC_Suppressed[faction_id] = {}
			FIZ_BuildUpdateList()
		end
	end
end

-----------------------------------
-- _11_ Prepare update entries	 --
-----------------------------------
function FIZ_ParseBagContent()	--TODO: This is basically just the MFAC.cache_bank_item code repeated
	FIZ_ItemsCarried = {}

	for bag = 0, NUM_BAG_SLOTS do
		local num = GetContainerNumSlots(bag)
		for slot = 1, num do
			local _icon, item_count, _locked, _quality, _readable, _lootable, _itemLink, _isFiltered, _noValue, item_id = GetContainerItemInfo(bag, slot);
			if (item_count and item_id) then
				local item_name = GetItemInfo(item_id)
				FIZ_ItemsCarried[item_name] = (FIZ_ItemsCarried[item_name] or 0) + item_count
			end
		end
	end
end

function MFAC.cache_bank_item(p_bank, p_container, p_slot)
	local _icon, item_count, _locked, _quality, _readable, _lootable, _itemLink, _isFiltered, _noValue, item_id = GetContainerItemInfo(p_container, p_slot);
	if (item_count and item_id) then
		local item_name = GetItemInfo(item_id)
		p_bank[item_name] = (p_bank[item_name] or 0) + item_count
	end
end

function MFAC.ParseBankContent()

	MFAC_Data.Bank[MFAC_Data.player_bank_key] = {}
	local player_bank = MFAC_Data.Bank[MFAC_Data.player_bank_key]

	local num = GetContainerNumSlots(BANK_CONTAINER)
	for j = 1, num do
		MFAC.cache_bank_item(player_bank, BANK_CONTAINER, j)
	end

	for i = (NUM_BAG_SLOTS + 1), (NUM_BAG_SLOTS + NUM_BANKBAGSLOTS) do
		local num_slots = GetContainerNumSlots(i)
		for j = 1, num_slots do
			MFAC.cache_bank_item(player_bank, BANK_CONTAINER, j)
		end
	end

--TODO: REAGENTBANK_CONTAINER = -3;

end


function FIZ_BuildUpdateList()
	FIZ_UpdateList = {}
	FIZ_CurrentRepInBag = 0
	FIZ_CurrentRepInBagBank = 0
	FIZ_CurrentRepInQuest = 0
	local index = 1
	local _
	local SPACER_LINE = {l = " "}

	if (not UI.ReputationDetailFrame:IsVisible()) then
		return
	end

	local faction_id = UI.ReputationDetailFrame.faction_id
	if(not faction_id or not FIZ_FactionGain[faction_id]) then
		MFAC.UpdateDetailList()
		return
	end

	local _name, _, standingId, _, barMax, barValue = GetFactionInfoByID(faction_id)

	local fg_sid=FIZ_FactionGain[faction_id][standingId]
	if (not fg_sid) then
		MFAC.UpdateDetailList()
		return;
	end

	local repToNext = barMax - barValue

	-- instances
	if (fg_sid.instance and MFAC_Data.ShowInstances) then
		local fg_sid_x=fg_sid.instance
		for i = 0, fg_sid.instance.count do
			local fg_sid_x_d=fg_sid_x.data[i]
			if (not fg_sid_x_d.limit or (barValue < fg_sid_x_d.limit)) then
				local toDo = string.format("%.1f", repToNext / fg_sid_x_d.rep)
				if (fg_sid_x_d.limit) then
					toDo = string.format("%.1f", (fg_sid_x_d.limit - barValue) / fg_sid_x_d.rep)
				end
				FIZ_UpdateList[index] = {}
				local FUL_I = FIZ_UpdateList[index]
				FUL_I.type = FIZ_TXT.instanceShort
				FUL_I.times = toDo.."x"
				FUL_I.rep = string.format("%d", fg_sid_x_d.rep)
				FUL_I.hasList = false
				FUL_I.listShown = nil
				FUL_I.index = index
				FUL_I.belongsTo = nil
				FUL_I.isShown = true
				FUL_I.name = fg_sid_x_d.name.." ("..fg_sid_x_d.level..")"

				FUL_I.tooltipHead = FIZ_TXT.instanceHead
				FUL_I.tooltipTip = FIZ_TXT.instanceTip

				FUL_I.tooltipDetails = {}
				local FUL_I_TD = FUL_I.tooltipDetails
				local x = 0
				FUL_I_TD[x] = {}
				FUL_I_TD[x].l = FIZ_TXT.instance2
				FUL_I_TD[x].r = fg_sid_x_d.name

				x = x+1
				FUL_I_TD[x] = {}
				FUL_I_TD[x].l = FIZ_TXT.mode
				FUL_I_TD[x].r = fg_sid_x_d.level
				x = x+1
				FUL_I_TD[x] = {}
				FUL_I_TD[x].l = FIZ_TXT.reputation
				FUL_I_TD[x].r = FUL_I.rep
				x = x+1
				FUL_I_TD[x] = {}
				FUL_I_TD[x].l = FIZ_TXT.timesToRun
				FUL_I_TD[x].r = FUL_I.times
				x = x+1
				FUL_I_TD[x] = {}
				FUL_I_TD[x].l = " "
				FUL_I_TD[x].r = " "
				x = x+1
				FUL_I_TD[x] = {}
				FUL_I_TD[x].l = FIZ_TXT.maxStanding
				FUL_I_TD[x].r = _G["FACTION_STANDING_LABEL"..fg_sid_x_d.maxStanding]
				FUL_I_TD.count = x
				FUL_I.tooltipDetails = FUL_I_TD
				FIZ_UpdateList[index] = FUL_I
				index = index + 1
			end
		end
	end

	-- mobs
	if (fg_sid.mobs and MFAC_Data.ShowMobs) then
		local fg_sid_x=fg_sid.mobs
		for i = 0, fg_sid_x.count do
			local fg_sid_x_d=fg_sid_x.data[i]
			if (not fg_sid_x_d.limit or (barValue < fg_sid_x_d.limit)) then
				local toDo = ceil(repToNext / fg_sid_x_d.rep)
				if (fg_sid_x_d.limit) then
					toDo = ceil((fg_sid_x_d.limit - barValue) / fg_sid_x_d.rep)
				end
				FIZ_UpdateList[index] = {}
				local FUL_I = FIZ_UpdateList[index]
				FUL_I.type = FIZ_TXT.mobShort
				FUL_I.times = toDo.."x"
				FUL_I.rep = string.format("%d", fg_sid_x_d.rep)
				FUL_I.hasList = false
				FUL_I.listShown = nil
				FUL_I.index = index
				FUL_I.belongsTo = nil
				FUL_I.isShown = true
				FUL_I.tooltipHead = FIZ_TXT.mobHead
				FUL_I.tooltipTip = FIZ_TXT.mobTip
				if (fg_sid_x_d.zone) then
					FUL_I.name = fg_sid_x_d.name.." ("..fg_sid_x_d.zone..")"
					FUL_I.tooltipDetails = {}
					local FUL_I_TD = FUL_I.tooltipDetails
					local x = 0
					FUL_I_TD[x] = {}
					FUL_I_TD[x].l = FIZ_TXT.mob2
					FUL_I_TD[x].r = fg_sid_x_d.name
					x = x+1
					FUL_I_TD[x] = {}
					FUL_I_TD[x].l = FIZ_TXT.location
					FUL_I_TD[x].r = fg_sid_x_d.zone
					x = x+1
					FUL_I_TD[x] = {}
					FUL_I_TD[x].l = FIZ_TXT.reputation
					FUL_I_TD[x].r = FUL_I.rep
					x = x+1
					FUL_I_TD[x] = {}
					FUL_I_TD[x].l = FIZ_TXT.toDo
					FUL_I_TD[x].r = FUL_I.times
					x = x+1
					FUL_I_TD[x] = {}
					FUL_I_TD[x].l = " "
					FUL_I_TD[x].r = " "
					x = x+1
					FUL_I_TD[x] = {}
					FUL_I_TD[x].l = FIZ_TXT.maxStanding
					FUL_I_TD[x].r = _G["FACTION_STANDING_LABEL"..fg_sid_x_d.maxStanding]
					FUL_I_TD.count = x
				else
					FUL_I.name = fg_sid_x_d.name
					FUL_I_TD = {}
					local x = 0
					FUL_I_TD[x] = {}
					FUL_I_TD[x].l = FIZ_TXT.mob2
					FUL_I_TD[x].r = fg_sid_x_d.name
					x = x+1
					FUL_I_TD[x] = {}
					FUL_I_TD[x].l = FIZ_TXT.reputation
					FUL_I_TD[x].r = FUL_I.rep
					x = x+1
					FUL_I_TD[x] = {}
					FUL_I_TD[x].l = FIZ_TXT.toDo
					FUL_I_TD[x].r = FUL_I.times
					x = x+1
					FUL_I_TD[x] = {}
					FUL_I_TD[x].l = " "
					FUL_I_TD[x].r = " "
					x = x+1
					FUL_I_TD[x] = {}
					FUL_I_TD[x].l = FIZ_TXT.maxStanding
					FUL_I_TD[x].r = _G["FACTION_STANDING_LABEL"..fg_sid_x_d.maxStanding]
					FUL_I_TD.count = x
					FUL_I.tooltipDetails = FUL_I_TD
				end
				FIZ_UpdateList[index] = FUL_I
				index = index + 1
			end
		end
	end

	-- items
	if (fg_sid.items and MFAC_Data.ShowItems) then
		local items = fg_sid.items
		for _k, item_entry in pairs(items) do
			local item_name = item_entry.name
			local item_rep = item_entry.rep
			local item_count = item_entry.count
			local bag_count = FIZ_ItemsCarried[item_name] or 0
			local bag_turn_ins = floor(bag_count / item_count)
			local num_needed = ceil(repToNext / item_rep)
			local times_txt = num_needed .. "x"
			local update_entry = {
				type = FIZ_TXT.itemsShort,
				times = times_txt,
				rep = tostring(item_rep),
				isShown = true,
				name = item_name,
				hasList = false,
				listShown = false,
				tooltipHead = FIZ_TXT.itemsHead,
				tooltipTip = FIZ_TXT.itemsTip,
				tooltipDetails = {
					{l = FIZ_TXT.itemsRequired},
					{l = times_txt, r = item_name},
				},
			}
			if (bag_turn_ins > 0) then
				update_entry.name = item_name .. MFAC.Colours.Bag .. " [x" .. bag_turn_ins .. "]|r"
				local rep_in_bag = bag_turn_ins * item_rep
				FIZ_CurrentRepInBag = FIZ_CurrentRepInBag + rep_in_bag

				tinsert(update_entry.tooltipDetails, SPACER_LINE)
				tinsert(update_entry.tooltipDetails, {l = FIZ_TXT.inBag})
				tinsert(update_entry.tooltipDetails, {l = "   " .. FIZ_TXT.turnIns, r = tostring(bag_turn_ins)})
				tinsert(update_entry.tooltipDetails, {l = "   " .. FIZ_TXT.reputation, r = tostring(rep_in_bag)})
			end
			tinsert(update_entry.tooltipDetails, SPACER_LINE)
			tinsert(update_entry.tooltipDetails, {l = FIZ_TXT.maxStanding, r = _G["FACTION_STANDING_LABEL" .. item_entry.maxStanding]})

			tinsert(FIZ_UpdateList, update_entry)

		end
	end

	-- General
	if (fg_sid.general and MFAC_Data.ShowGeneral) then
		local fg_sid_x=fg_sid.general
		for i = 0, fg_sid_x.count do
			local fg_sid_x_d=fg_sid_x.data[i]
			if (not fg_sid_x_d.limit or (barValue < fg_sid_x_d.limit)) then
				local toDo = string.format("%.1f", repToNext / fg_sid_x_d.rep)
				if (fg_sid_x_d.limit) then
					toDo = string.format("%.1f", (fg_sid_x_d.limit - barValue) / fg_sid_x_d.rep)
				end
				FIZ_UpdateList[index] = {}
				local FUL_I = FIZ_UpdateList[index]
				FUL_I.type = FIZ_TXT.generalShort
				FUL_I.times = toDo.."x"
				FUL_I.rep = string.format("%d", fg_sid_x_d.rep)
				FUL_I.hasList = false
				FUL_I.listShown = nil
				FUL_I.index = index
				FUL_I.belongsTo = nil
				FUL_I.isShown = true
				FUL_I.name = fg_sid_x_d.name

				if (fg_sid_x_d.head and fg_sid_x_d.head ~= "") then
					FUL_I.tooltipHead = fg_sid_x_d.head
				else
					FUL_I.tooltipHead = FIZ_TXT.generalHead
				end
				if (fg_sid_x_d.tip and fg_sid_x_d.tip ~= "") then
					FUL_I.tooltipTip = fg_sid_x_d.tip
				else
					FUL_I.tooltipTip = FIZ_TXT.generalTip
				end

				FUL_I.tooltipDetails = {}
				local FUL_I_TD = FUL_I.tooltipDetails
				local x = 0
				FUL_I_TD[x] = {}
				FUL_I_TD[x].l = FIZ_TXT.general2
				FUL_I_TD[x].r = fg_sid_x_d.name
				x = x+1
				FUL_I_TD[x] = {}
				FUL_I_TD[x].l = FIZ_TXT.reputation
				FUL_I_TD[x].r = FUL_I.rep
				x = x+1
				FUL_I_TD[x] = {}
				FUL_I_TD[x].l = FIZ_TXT.timesToRun
				FUL_I_TD[x].r = FUL_I.times
				x = x+1
				FUL_I_TD[x] = {}
				FUL_I_TD[x].l = " "
				FUL_I_TD[x].r = " "

				if (fg_sid_x_d.tipList) then
					for tip in pairs(fg_sid_x_d.tipList) do
						x = x+1
						FUL_I_TD[x] = {}
						FUL_I_TD[x].l = tip
						FUL_I_TD[x].r = fg_sid_x_d.tipList[tip]
					end
					x = x+1
					FUL_I_TD[x] = {}
					FUL_I_TD[x].l = " "
					FUL_I_TD[x].r = " "
				end

				x = x+1
				FUL_I_TD[x] = {}
				FUL_I_TD[x].l = FIZ_TXT.maxStanding
				FUL_I_TD[x].r = _G["FACTION_STANDING_LABEL"..fg_sid_x_d.maxStanding]
				FUL_I_TD.count = x
				local FUL_I_TD = FUL_I.tooltipDetails
				FIZ_UpdateList[index] = FUL_I
				index = index + 1
			end
		end
	end

	-- quests (may have items)
	local sum = 0
	local count = 0
	if (fg_sid.quest_list and MFAC_Data.ShowQuests) then
		local quest_list = fg_sid.quest_list
		for i = 0, quest_list.count do
			local fg_sid_x_d = quest_list.data[i]
			local showQuest = true
			if (fg_sid_x_d.profession) then
				local profession = fg_sid_x_d.profession
				showQuest = bit.band(profession, MFAC.SkillTrackBits) > 0
			end

			if (showQuest) then
				if (not fg_sid_x_d.limit or (barValue < fg_sid_x_d.limit)) then
					local toDo = ceil(repToNext / fg_sid_x_d.rep)
					if (fg_sid_x_d.limit) then
						toDo = ceil((fg_sid_x_d.limit - barValue) / fg_sid_x_d.rep)
					end
					FIZ_UpdateList[index] = {}
					local FUL_I = FIZ_UpdateList[index]
					FUL_I.type = FIZ_TXT.questShort
					FUL_I.times = toDo .. "x"
					FUL_I.rep = string.format("%d", fg_sid_x_d.rep)
					FUL_I.index = index
					FUL_I.belongsTo = nil
					FUL_I.isShown = true
					FUL_I.name = fg_sid_x_d.name
					FUL_I.originalName = FUL_I.name
					FUL_I.faction = faction_id
					FUL_I.canSuppress = true
					FUL_I.suppress = nil
					if (MFAC_Suppressed and MFAC_Suppressed[faction_id] and MFAC_Suppressed[faction_id][FUL_I.originalName]) then
						FUL_I.suppress = true
					end
					FUL_I.tooltipHead = FIZ_TXT.questHead
					FUL_I.tooltipTip = FIZ_TXT.questTip

					FUL_I.tooltipDetails = {}
					local FUL_I_TD = FUL_I.tooltipDetails
					local x = 0
					FUL_I_TD[x] = {}
					FUL_I_TD[x].l = FIZ_TXT.quest2
					FUL_I_TD[x].r = FUL_I.name
					x = x + 1
					FUL_I_TD[x] = {}
					FUL_I_TD[x].l = FIZ_TXT.reputation
					FUL_I_TD[x].r = FUL_I.rep
					x = x + 1
					FUL_I_TD[x] = {}
					FUL_I_TD[x].l = FIZ_TXT.timesToDo
					FUL_I_TD[x].r = FUL_I.times
					x = x + 1

					if (not FUL_I.suppress) then
						sum = sum + fg_sid_x_d.rep
						count = count + 1
					end

					if (fg_sid_x_d.items) then
						FUL_I.hasList = true
						FUL_I.listShown = false

						FUL_I_TD[x] = {}
						FUL_I_TD[x].l = " "
						FUL_I_TD[x].r = " "
						x = x + 1
						FUL_I_TD[x] = {}
						FUL_I_TD[x].l = FIZ_TXT.itemsRequired
						FUL_I_TD[x].r = " "
						x = x + 1

						-- quest In log?
						FUL_I.lowlight = nil

						-- check if this quest is known
						local entries, _quests = MFAC.GetNumQuestLogEntries()
						for z = 1, entries do
							local title, header = MFAC.GetQuestInfoForIndex(z)
							if (title and not header) then
								if string.find(string.lower(fg_sid_x_d.name), string.lower(title)) then
									-- this quest matches
									FUL_I.lowlight = true
									FUL_I.name = FUL_I.name .. MFAC.Colours.ActiveQuest .. " (" .. FIZ_TXT.active .. ")|r"
								end
							end
						end

						-- add items
						local itemIndex = index + 1

						local currentQuestTimesBag = -1
						local currentQuestTimesBagBank = -1
						for item in pairs(fg_sid_x_d.items) do
							FIZ_UpdateList[itemIndex] = {}
							local FUL_II = FIZ_UpdateList[itemIndex]
							FUL_II.type = ""
							FUL_II.times = (fg_sid_x_d.items[item] * toDo) .. "x"
							FUL_II.rep = nil
							FUL_II.index = itemIndex
							FUL_II.belongsTo = index
							FUL_II.hasList = nil
							FUL_II.listShown = nil
							FUL_II.isShown = FUL_I.listShown
							FUL_II.name = item .. " (" .. fg_sid_x_d.items[item] .. "x)"

							FUL_I_TD[x] = {}
							FUL_I_TD[x].l = fg_sid_x_d.items[item] .. "x"
							FUL_I_TD[x].r = item
							x = x + 1

							if (FIZ_ItemsCarried and FIZ_ItemsCarried[item]) then
								if ((FIZ_ItemsCarried[item] >= fg_sid_x_d.items[item]) and (fg_sid_x_d.items[item] > 0)) then
									FUL_II.name =
										FUL_II.name .. MFAC.Colours.Bag .. " [" .. FIZ_ItemsCarried[item] .. "x]|r"
									FUL_II.currentTimesBag = floor(FIZ_ItemsCarried[item] / fg_sid_x_d.items[item])
									if (currentQuestTimesBag == -1) then
										-- first items for this quest --> take value
										currentQuestTimesBag = FUL_II.currentTimesBag
									else
										-- some items already Set
										if (FUL_II.currentTimesBag < currentQuestTimesBag) then
											-- fewer of this item than of others, reduce quest count
											currentQuestTimesBag = FUL_II.currentTimesBag
										end
									end
								else
									-- not enough of this item for quest -> set to 0
									currentQuestTimesBag = 0
									FUL_II.name = FUL_II.name .. " [" .. FIZ_ItemsCarried[item] .. "x]"
								end
								if (MFAC_Data.Bank[MFAC_Data.player_bank_key][item]) then
									local total = MFAC_Data.Bank[MFAC_Data.player_bank_key][item] + FIZ_ItemsCarried[item]
									if ((total >= fg_sid_x_d.items[item]) and (fg_sid_x_d.items[item] > 0)) then
										FUL_II.name = FUL_II.name .. MFAC.Colours.Bank .. " [" .. total .. "x]|r"
										FUL_II.currentTimesBagBank = floor(total / fg_sid_x_d.items[item])
										if (currentQuestTimesBagBank == -1) then
											-- first items for this quest --> take value
											currentQuestTimesBagBank = FUL_II.currentTimesBagBank
										else
											-- some items already Set
											if (FUL_II.currentTimesBagBank < currentQuestTimesBagBank) then
												-- fewer of this item than of others, reduce quest count
												currentQuestTimesBagBank = FUL_II.currentTimesBagBank
											end
										end
									else
										-- not enough of this item for quest -> set to 0
										currentQuestTimesBagBank = 0
										FUL_II.name = FUL_II.name .. " [" .. total .. "x]"
									end
								else
									-- none of this carried In bank
								end
							else
								-- not enough of this item for quest -> set to 0
								currentQuestTimesBag = 0
							end
							FIZ_UpdateList[itemIndex] = FUL_II
							itemIndex = itemIndex + 1
						end
						if (currentQuestTimesBag > 0) then
							FUL_I.name = FUL_I.name .. MFAC.Colours.Bag .. " [" .. currentQuestTimesBag .. "x]|r"
							FUL_I.currentTimesBag = currentQuestTimesBag
							FUL_I.currentRepBag = currentQuestTimesBag * FUL_I.rep
							FUL_I.highlight = true
							FUL_I.name = FUL_I.originalName
							FUL_I.lowlight = nil
							FIZ_CurrentRepInBag = FIZ_CurrentRepInBag + FUL_I.currentRepBag

							FUL_I_TD[x] = {}
							FUL_I_TD[x].l = " "
							FUL_I_TD[x].r = " "
							x = x + 1
							FUL_I_TD[x] = {}
							FUL_I_TD[x].l = FIZ_TXT.inBag
							FUL_I_TD[x].r = " "
							x = x + 1
							FUL_I_TD[x] = {}
							FUL_I_TD[x].l = FIZ_TXT.turnIns
							FUL_I_TD[x].r = string.format("%d", FUL_I.currentTimesBag)
							x = x + 1
							FUL_I_TD[x] = {}
							FUL_I_TD[x].l = FIZ_TXT.reputation
							FUL_I_TD[x].r = string.format("%d", FUL_I.currentRepBag)
							x = x + 1
						else
							FUL_I.currentTimesBag = nil
							FUL_I.currentRepBag = nil
						end
						if (currentQuestTimesBagBank > 0) then
							FUL_I.name = FUL_I.name .. MFAC.Colours.Bank .. " [" .. currentQuestTimesBagBank .. "x]|r"
							FUL_I.currentTimesBagBank = currentQuestTimesBagBank
							FUL_I.currentRepBagBank = currentQuestTimesBagBank * FUL_I.rep
							FUL_I.highlight = true
							FUL_I.name = FUL_I.originalName
							FUL_I.lowlight = nil
							FIZ_CurrentRepInBagBank = FIZ_CurrentRepInBagBank + FUL_I.currentRepBagBank

							FUL_I_TD[x] = {}
							FUL_I_TD[x].l = " "
							FUL_I_TD[x].r = " "
							x = x + 1
							FUL_I_TD[x] = {}
							FUL_I_TD[x].l = FIZ_TXT.inBagBank
							FUL_I_TD[x].r = " "
							x = x + 1
							FUL_I_TD[x] = {}
							FUL_I_TD[x].l = FIZ_TXT.turnIns
							FUL_I_TD[x].r = string.format("%d", FUL_I.currentTimesBagBank)
							x = x + 1
							FUL_I_TD[x] = {}
							FUL_I_TD[x].l = FIZ_TXT.reputation
							FUL_I_TD[x].r = string.format("%d", FUL_I.currentRepBagBank)
							x = x + 1
						else
							FUL_I.currentTimesBagBank = nil
							FUL_I.currentRepBagBank = nil
						end
						if ((currentQuestTimesBag == 0) and (currentQuestTimesBagBank)) then
							FUL_I.highlight = nil
						end

						FUL_I_TD[x] = {}
						FUL_I_TD[x].l = " "
						FUL_I_TD[x].r = " "
						x = x + 1
						FUL_I_TD[x] = {}
						FUL_I_TD[x].l = FIZ_TXT.maxStanding
						FUL_I_TD[x].r = _G["FACTION_STANDING_LABEL" .. fg_sid_x_d.maxStanding]
						x = x + 1

						FUL_I_TD.count = x - 1
						index = itemIndex
					else
						-- no items to add
						FUL_I.hasList = false
						FUL_I.listShown = nil
						FUL_I.highlight = nil -- will be Changed below if needed
						FUL_I.lowlight = nil

						-- check if this quest is known and/or completed
						local entries, _quests = MFAC.GetNumQuestLogEntries()
						for z = 1, entries do
							local title, header, complete = MFAC.GetQuestInfoForIndex(z)
							if (title and not header) then
								if string.find(string.lower(fg_sid_x_d.name), string.lower(title)) then
									-- this quest matches
									if (complete) then
										FUL_I.highlight = true
										FUL_I.name = FUL_I.name .. MFAC.Colours.Quest .. " (" .. FIZ_TXT.complete .. ")|r"
										FUL_I.currentTimesQuest = 1
										FUL_I.currentRepQuest = FUL_I.rep

										FIZ_CurrentRepInQuest = FIZ_CurrentRepInQuest + fg_sid_x_d.rep

										FUL_I_TD[x] = {}
										FUL_I_TD[x].l = " "
										FUL_I_TD[x].r = " "
										x = x + 1
										FUL_I_TD[x] = {}
										FUL_I_TD[x].l = FIZ_TXT.questCompleted
										FUL_I_TD[x].r = " "
										x = x + 1
										FUL_I_TD[x] = {}
										FUL_I_TD[x].l = FIZ_TXT.reputation
										FUL_I_TD[x].r = string.format("%d", FUL_I.currentRepQuest)
										x = x + 1
									else
										FUL_I.lowlight = true
										FUL_I.name =
											FUL_I.name .. MFAC.Colours.ActiveQuest .. " (" .. FIZ_TXT.active .. ")|r"
									end
								end
							end
						end

						FUL_I_TD[x] = {}
						FUL_I_TD[x].l = " "
						FUL_I_TD[x].r = " "
						x = x + 1
						FUL_I_TD[x] = {}
						FUL_I_TD[x].l = FIZ_TXT.maxStanding
						FUL_I_TD[x].r = _G["FACTION_STANDING_LABEL" .. fg_sid_x_d.maxStanding]
						x = x + 1

						FUL_I_TD.count = x - 1
						FIZ_UpdateList[index] = FUL_I
						index = index + 1
					end
				end
			end

			if ((sum > 0) and (count > 1)) then
				-- add virtual quest to show summary of all quests:
				local toDo = ceil(repToNext / sum)
				FIZ_UpdateList[index] = {}
				local FUL_I = FIZ_UpdateList[index]
				FUL_I.type = FIZ_TXT.questShort
				FUL_I.times = toDo .. "x"
				FUL_I.rep = string.format("%d", sum)
				FUL_I.index = index
				FUL_I.belongsTo = nil
				FUL_I.isShown = true
				FUL_I.name = string.format(FIZ_TXT.allOfTheAbove, count)
				FUL_I.tooltipHead = string.format(FIZ_TXT.questSummaryHead, count)
				FUL_I.tooltipTip = FIZ_TXT.questSummaryTip

				FUL_I_TD = {}
				local x = 0
				FUL_I_TD[x] = {}
				FUL_I_TD[x].l = FIZ_TXT.reputation
				FUL_I_TD[x].r = FUL_I.rep
				x = x + 1
				FUL_I_TD[x] = {}
				FUL_I_TD[x].l = FIZ_TXT.timesToDo
				FUL_I_TD[x].r = FUL_I.times
				FUL_I_TD.count = x
				FUL_I.tooltipDetails = FUL_I_TD
				FIZ_UpdateList[index] = FUL_I
				index = index + 1
			end
		end
	end




	--MFAC.Print("Added "..(index-1).." entries for ["..faction.."] at standing "..standingId)

	MFAC.UpdateDetailList()

end

function FIZ_GetUpdateListSize()
	local count = 0
	local highest = 0
	for i in pairs(FIZ_UpdateList) do
		if (FIZ_UpdateList[i].isShown) then
			count = count + 1
			if (i > highest) then
				highest = i
			end
		end
	end

	return count, highest
end

function FIZ_ShowUpdateEntry(index, show)
	if (not FIZ_UpdateList[index]) then return end		-- invalid index
	if (not FIZ_UpdateList[index].hasList) then return end	-- not a list Header entry
	if (type(show)~="boolean") then return end		-- wrong data type

	FIZ_UpdateList[index].listShown = show
	for i in pairs(FIZ_UpdateList) do
		if (FIZ_UpdateList[i].belongsTo == index) then
			FIZ_UpdateList[i].isShown = show
		end
	end

	MFAC.UpdateDetailList()

end

function FIZ_ShowUpdateEntries(show)
	if (type(show)~="boolean") then return end		-- wrong data type

	for i in pairs(FIZ_UpdateList) do
		if (FIZ_UpdateList[i].belongsTo == nil) then
			-- always show parent entries, show or Hide their children
			FIZ_UpdateList[i].isShown = true
			FIZ_UpdateList[i].listShown = show
		else
			-- show or Hide child entries
			FIZ_UpdateList[i].isShown = show
		end
	end

	MFAC.UpdateDetailList()

end

function FIZ_ShowLineToolTip(self)
	if not self then return end

	if (self.tooltipHead) then
		GameTooltip_SetDefaultAnchor(GameTooltip, self)
		GameTooltip:SetText(self.tooltipHead, 1, 1, 0.5, 1)
		if (self.tooltipTip) then
			GameTooltip:AddLine(self.tooltipTip, 1, 1, 1, 1)
		end
		if (self.tooltipDetails and type(self.tooltipDetails) == "table") then
			GameTooltip:AddLine(" ", 1, 1, 1, 1)
			for _i, line in ipairs(self.tooltipDetails) do
				if (line.l or line.r) then
					if (not line.r) then
						GameTooltip:AddLine(line.l, 1, 1, 0, 1)
					else
						GameTooltip:AddDoubleLine(line.l, line.r, 1, 1, 0.5, 1, 1, 1)
					end
				end
			end
		end
		GameTooltip:Show()
	end
end

function FIZ_ShowHelpToolTip(self, p_element_name)
	if not p_element_name then return end

	local eletxt = FIZ_TXT.elements;
	local tip = ""
	local head = ""
	if (eletxt.name[p_element_name] and eletxt.tip[p_element_name]) then

		tip = eletxt.tip[p_element_name]
		head = eletxt.name[p_element_name]

		if (MFAC.missing_toolip and MFAC.missing_toolip[p_element_name]) then
			MFAC.missing_toolip[p_element_name] = nil
		end
	else
		if (not MFAC.missing_toolip) then
			MFAC.missing_toolip = {}
		end
		MFAC.missing_toolip[p_element_name] = true
	end

	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	if (head ~= "") then
		GameTooltip:SetText(head, 1, 1, 0.5, 1.0, 1)
		GameTooltip:AddLine(tip, 1, 1, 1, 1.0, 1)
	end

	GameTooltip:SetMinimumWidth(300);
	GameTooltip:Show()

	if(MFAC.missing_toolip) then
		MFAC.Print("|n|n");
		for k, _v in pairs(MFAC.missing_toolip) do
			MFAC.Print("   Missing tooltip:" .. k);
		end
	end
end

-----------------------------------
-- _12_ reputation Changes to chat
-----------------------------------

-- Track if the faction ever was Paragon
--Sometimes, notably when entering Withered Training, the WoW api fails to properly report Paragon status
--If you ever were Paragon, then you still are (as far as the current rules go, anyway)

--MFAC_Data.tracked_faction = 1859;

function MFAC.get_rep_change(p_faction_id, p_faction_name, p_stored_rep, p_bar_min, p_bar_max, p_bar_current, p_stored_paragon_rep, p_paragon_rep, p_paragon_threshold, p_old_standing, p_new_standing) --rep_delta, rep_left, rep_current
	local is_paragon = p_faction_id and MFAC.IsFactionParagon(p_faction_id);

	if(p_faction_id == MFAC_Data.tracked_faction) then
		print("logging info for:", p_faction_name);
		print("  p_stored_rep", p_stored_rep, "p_bar_min", p_bar_min, "p_bar_max", p_bar_max, "p_bar_current", p_bar_current)
		print("  is paragon:", is_paragon, "was paragon", MFAC.StoredRep[p_faction_name].paragon);
	end

	--HACK
	-- If you are paragon, record that for the elseif check on future calls
	-- If you're not Paragon, but you were and you have a valid faction id, then the API is returning nonsense so bail out
	if(is_paragon) then
		MFAC.StoredRep[p_faction_name].paragon = true;
	elseif (p_faction_id and MFAC.StoredRep[p_faction_name].paragon) then
		return nil;
	end
	--END HACK

	local rep_delta, rep_left, rep_current;

	if(not is_paragon) then
if(p_faction_id == MFAC_Data.tracked_faction) then print("not is_paragon"); end;
		rep_delta = p_bar_current - p_stored_rep
		rep_current = p_bar_current;
		rep_left = p_bar_max - p_bar_current;
		return rep_delta, rep_left, rep_current;
	end

if(p_faction_id == MFAC_Data.tracked_faction) then
	print(" paragon_rep:", p_paragon_rep, "threshold:", p_paragon_threshold, "stored:", p_stored_rep);
end;

	if(p_stored_rep ~= p_bar_max) then --We just hit Exalted with a paragon faction, so it wrapped around
		rep_delta = p_bar_current - p_stored_rep + p_paragon_rep;
	else
		rep_delta = p_paragon_rep - p_stored_paragon_rep;
	end

	rep_current = p_paragon_rep;

	local rep_remainder = (p_paragon_rep % p_paragon_threshold);
	rep_left = p_paragon_threshold - rep_remainder;

	return rep_delta, rep_left, rep_current;


end

function MFAC.should_write_chat_message(p_faction_name)

	if(not MFAC_Data.WriteChatMessage) then
		return false;
	end;

	if((p_faction_name == MFAC.GuildName) and MFAC_Data.NoGuildGain) then
		return false;
	end

	return true;
end

function MFAC.should_switch_faction_bar(p_faction_name)

	if(not MFAC_Data.SwitchFactionBar) then
		return false
	end

	if(p_faction_name == MFAC.GuildName and MFAC_Data.NoGuildSwitch) then
		return false;
	end

	return true;
end

function MFAC.InitFactionRepStore(p_faction_id, p_name)

	local _, _, standingID, _, _, barValue = GetFactionInfoByID(p_faction_id)

	local paragon_rep = MFAC.GetFactionParagonInfo(p_faction_id);

	MFAC.StoredRep[p_name] = {}
	MFAC.StoredRep[p_name].session_gain = 0
	MFAC.StoredRep[p_name].standingID = standingID
	MFAC.StoredRep[p_name].rep = barValue
	if(paragon_rep ~= nil) then
		MFAC.StoredRep[p_name].paragon_rep = paragon_rep;
		MFAC.StoredRep[p_name].paragon = true;
	end

	if(p_faction_id == MFAC_Data.tracked_faction) then print(" Initialized ", p_name, " rep:", MFAC.StoredRep[p_name].rep, " paragon rep:", MFAC.StoredRep[p_name].paragon_rep); end;

end

function MFAC.InitRepStore()
	local name, factionID
	local numFactions = GetNumFactions();
	local _

	for factionIndex = 1, numFactions do
		name, _, _, _, _, _, _, _, _, _, _, _, _, factionID = GetFactionInfo(factionIndex)

		if(factionID and not MFAC.StoredRep[name]) then
			MFAC.InitFactionRepStore(factionID, name);
		end
	end
end


function FIZ_DumpReputationChangesToChat()
	local numFactions = GetNumFactions();

	local watchIndex = nil
	local watchedIndex = 0
	local gender = UnitSex("player");
	local _

	local name, standingID, barMin, barMax, barValue, isHeader, hasRep, isWatched, factionID
	for factionIndex=1, numFactions, 1 do
		name, _, standingID, barMin, barMax, barValue, _, _, isHeader, _, hasRep, isWatched, _, factionID = GetFactionInfo(factionIndex)
		if(not factionID) then
			break;
		end
		local is_paragon = factionID and MFAC.IsFactionParagon(factionID);
		local paragon_rep, paragon_threshold = MFAC.GetFactionParagonInfo(factionID);

		if(not MFAC.StoredRep[name]) then
			MFAC.InitFactionRepStore(factionID, name)
		end

		if (not isHeader or hasRep) then
			if (isWatched) then
				watchedIndex = factionIndex
			end
			local rep_delta, rep_left, rep_current = MFAC.get_rep_change(factionID, name, MFAC.StoredRep[name].rep, barMin, barMax, barValue, MFAC.StoredRep[name].paragon_rep, paragon_rep, paragon_threshold, MFAC.StoredRep[name].standingID, standingID)
if(factionID == MFAC_Data.tracked_faction) then print(" delta:",rep_delta, "  RepLeft:", rep_left, "  CurrentRep:", rep_current); end;
			if(rep_delta == nil) then
				print("MuffinFactionizer: WoW API is returning nonsense. Ignoring it.")
				return;
			end
			MFAC.StoredRep[name].session_gain = MFAC.StoredRep[name].session_gain + rep_delta;
			if (MFAC.should_write_chat_message(name)) then
				local standing_id = MuffinFactionizer.standing
				local current_standing_name, next_standing_name
				if (standingID < standing_id.Exalted) then
					current_standing_name = GetText("FACTION_STANDING_LABEL"..standingID, gender);
					next_standing_name = GetText("FACTION_STANDING_LABEL"..(standingID + 1), gender);
				elseif(standingID == standing_id.Exalted and not is_paragon) then
					current_standing_name = GetText("FACTION_STANDING_LABEL"..standingID, gender);
				elseif(standingID == standing_id.Exalted or standingID == standing_id.Paragon) then
					current_standing_name = "Paragon";
				end


				local sign = ""
				if (rep_delta > 0) then
					-- increased rep
					local dump_string = MFAC.Colours.RepGain .. string.format(FACTION_STANDING_INCREASED .. FIZ_TXT.stats, name, rep_delta, sign, MFAC.StoredRep[name].session_gain, rep_left)
					dump_string = dump_string .. " (" .. current_standing_name
					if(next_standing_name) then
						dump_string = dump_string .. " -> " .. next_standing_name
					end
					dump_string = dump_string .. ")"
					MFAC.Print(dump_string)
				elseif (rep_delta < 0) then -- decreased rep
					MFAC.Print(MFAC.Colours.RepLoss .. string.format(FACTION_STANDING_DECREASED .. FIZ_TXT.stats, name, -1 * rep_delta, sign, MFAC.StoredRep[name].session_gain, rep_left))
				end

				if (MFAC.StoredRep[name].standingID ~= standingID) then
					MFAC.Print(MFAC.Colours.NewStanding .. string.format(FACTION_STANDING_CHANGED, _G["FACTION_STANDING_LABEL" .. standingID], name))
				end
			end

			-- Update stored standing and reps
			MFAC.StoredRep[name].standingID = standingID
			MFAC.StoredRep[name].rep = barValue
			MFAC.StoredRep[name].paragon_rep = paragon_rep;

			if (MFAC.should_switch_faction_bar(name) and rep_delta > 0) then
				watchIndex = factionIndex
			end
		end
	end

	if (watchIndex and (watchIndex ~= watchedIndex) and (watchIndex ~= MuffinFactionizer.set_watch_idx)) then
		MuffinFactionizer.set_watch_idx = watchIndex
		--If we're not in combat, delay a little bit so we don't double report the switch. There is some lag in Blizzard's api
		--If we are in combat then MFAC.update_faction_watch will be called by the PLAYER_REGEN event
		if(not InCombatLockdown()) then
			C_Timer.After(MuffinFactionizer.BarSwitchDelay, function()  MFAC.update_faction_watch();  end)
		end
	end
end

function FIZ_ClearSessionGain()
	local factionIndex = GetSelectedFaction()
	local name = GetFactionInfo(factionIndex)

	if (name) then
		if(not MFAC.StoredRep[name]) then
			MFAC.StoredRep[name] = {}
		end

		MFAC.StoredRep[name].session_gain = 0
	end
	MFAC.ReputationFrame_Update()
end

-----------------------------------
-- _13_ chat filtering
-----------------------------------

-- AZERITE_ITEM_POWER_LEVEL_CHANGED (table), 13,14, (table)
-- AZERITE_ITEM_EXPERIENCE_CHANGED <table> 5187, 5

--CHAT_MSG_SYSTEM  Hear of Azeroth gains 168 Artifact Power.  Format string is ARTIFACT_XP_GAIN
-- AZERITE_POWER_TOOLTIP_TITLE is "Level %s (%s Artifact Power to next level)";
function FIZ_ChatFilter(_chatFrame, event, ...) -- chatFrame = self

--	CHAT_MSG_COMBAT_FACTION_CHANGE
--		Fires when player's faction changes. ie: "Your reputation with Timbermaw Hold has very slightly increased." -- NEW 1.9
--		arg1
--			The message to display
--
--	COMBAT_TEXT_UPDATE
--		arg1
--			Combat message type.
--			Known values include "HONOR_GAINED", and "FACTION".
--		arg2
--			for faction gain, this is the faction name.
--		arg3
--			for faction gain, the amount of reputation gained.


	local msg, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13 = ...;
	local skip = false
	if (MFAC_Data.SuppressOriginalChat and event) then

		if (event == "CHAT_MSG_COMBAT_FACTION_CHANGE") then
			skip = true
		end
		if ((event == "COMBAT_TEXT_UPDATE") and (msg=="FACTION")) then
			skip = true
		end

	end
	return skip, msg, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13
end

-----------------------------------
-- _13_ show option window
-----------------------------------
function FIZ_ToggleConfigWindow()
	--- fpt f_tcw	FIZ_Printtest("f_tcw","","")
	if ReputationFrame:IsVisible() then
		if UI.ReputationOptionsFrame:IsVisible() then
			-- both windows shown -> hide them both
			UI.ReputationOptionsFrame:Hide()
			HideUIPanel(CharacterFrame)
		else
			-- options window not shown -> show, hide any detail window
			UI.ReputationOptionsFrame:Show()
			UI.ReputationDetailFrame:Hide();
			ReputationDetailFrame:Hide();
		end
	else
		-- window not shown -> show both
		ToggleCharacter("ReputationFrame")
		UI.ReputationDetailFrame:Hide();
		ReputationDetailFrame:Hide();
		UI.ReputationOptionsFrame:Show()
	end
end

function FIZ_ToggleDetailWindow()
	--- fpt f_tdw	FIZ_Printtest("f_tdw","","")
	if ReputationFrame:IsVisible() then
		if (MFAC_Data.ExtendDetails) then
			if UI.ReputationDetailFrame:IsVisible() then
				-- both windows shown -> hide them both
				UI.ReputationDetailFrame:Hide()
				HideUIPanel(CharacterFrame)
			else
				-- detail window not shown -> show it, hide any others
				UI.ReputationDetailFrame:Show()
				ReputationDetailFrame:Hide();
				UI.ReputationOptionsFrame:Hide();
				ReputationFrame_Update(); -- rfl Event
			end
		else
			if ReputationDetailFrame:IsVisible() then
				-- both windows shown -> hide them both
				ReputationDetailFrame:Hide()
				HideUIPanel(CharacterFrame)
			else
				-- detail window not shown -> show it, hide any others
				UI.ReputationDetailFrame:Hide()
				UI.ReputationOptionsFrame:Hide();
				ReputationDetailFrame:Show();
				ReputationFrame_Update();
			end
		end
	else
		-- window not shown -> show both
		ToggleCharacter("ReputationFrame")
		if (MFAC_Data.ExtendDetails) then
			UI.ReputationDetailFrame:Show();
		else
			ReputationDetailFrame:Show();
		end
		UI.ReputationOptionsFrame:Hide()
		ReputationFrame_Update(); -- rfl Event
	end
end


-------------------------------------------
-- _15_ Getting reputation ready to hand In
-------------------------------------------
function FIZ_GetReadyReputation(p_faction_data)

	assert(type(p_faction_data) == "table")

	local result = 0
	local faction = p_faction_data.faction;
	local faction_id = faction.faction_id;
	local standing_id = faction.standing_id;

	if (faction.is_header) then
		return 0;
	end

	if (faction.faction_id) then
		local repToNext = faction.bar_max - faction.bar_value;

		local FIZ_FG_f = FIZ_FactionGain[faction_id]
		if (FIZ_FG_f) then
		local FIZ_FG_fs=FIZ_FG_f[standing_id]
			if (FIZ_FG_fs) then

				-- quests (may have items)
				local FIZ_FG_fs_h=FIZ_FG_fs.quests
				if (FIZ_FG_fs_h) then
					for i = 0, FIZ_FG_fs_h.count do
					local FIZ_FG_fs_h_d=FIZ_FG_fs_h.data[i]
						if (not FIZ_FG_fs_h_d.limit or (faction.bar_value < FIZ_FG_fs_h_d.limit)) then
							local toDo = ceil(repToNext / FIZ_FG_fs_h_d.rep)
							if (FIZ_FG_fs_h_d.limit) then
								toDo = ceil((FIZ_FG_fs_h_d.limit - faction.bar_value) / FIZ_FG_fs_h_d.rep)
							end

							if (FIZ_FG_fs_h_d.items) then
								local currentQuestTimesBag = -1
								local currentQuestTimesBagBank = -1
								for item in pairs(FIZ_FG_fs_h_d.items) do
									if (FIZ_ItemsCarried and FIZ_ItemsCarried[item]) then
										if ((FIZ_ItemsCarried[item] >= FIZ_FG_fs_h_d.items[item]) and (FIZ_FG_fs_h_d.items[item] > 0)) then
											local localCurrentTimesBag = floor(FIZ_ItemsCarried[item] / FIZ_FG_fs_h_d.items[item])
											if (currentQuestTimesBag == -1) then
												-- first items for this quest --> take value
												currentQuestTimesBag = localCurrentTimesBag
											else
												-- some items already Set
												if (localCurrentTimesBag < currentQuestTimesBag) then
													-- fewer of this item than of others, reduce quest count
													currentQuestTimesBag = localCurrentTimesBag
												end
											end
										else
											-- not enough of this item for quest -> set to 0
											currentQuestTimesBag = 0
										end
										if (MFAC_Data.Bank[MFAC_Data.player_bank_key][item]) then
											local total = MFAC_Data.Bank[MFAC_Data.player_bank_key][item] + FIZ_ItemsCarried[item]
											if ((total >= FIZ_FG_fs_h_d.items[item]) and (FIZ_FG_fs_h_d.items[item] > 0)) then
												local localCurrentTimesBagBank = floor(total / FIZ_FG_fs_h_d.items[item])
												if (currentQuestTimesBagBank == -1) then
													-- first items for this quest --> take value
													currentQuestTimesBagBank = localCurrentTimesBagBank
												else
													-- some items already Set
													if (localCurrentTimesBagBank < currentQuestTimesBagBank) then
														-- fewer of this item than of others, reduce quest count
														currentQuestTimesBagBank = localCurrentTimesBagBank
													end
												end
											else
												-- not enough of this item for quest -> set to 0
												currentQuestTimesBagBank = 0
											end
										else
											-- none of this carried In bank
										end
									else
										-- not enough of this item for quest -> set to 0
										currentQuestTimesBag = 0
									end
								end
								if (currentQuestTimesBag > toDo) then
									currentQuestTimesBag = toDo
								end
								if (currentQuestTimesBagBank > toDo) then
									currentQuestTimesBagBank = toDo
								end
								if (currentQuestTimesBagBank > 0) then
									result = result + currentQuestTimesBagBank * FIZ_FG_fs_h_d.rep
								elseif (currentQuestTimesBag > 0) then
									result = result + currentQuestTimesBag * FIZ_FG_fs_h_d.rep
								else
									-- nothing to add
								end
							else
								-- no items, check if this quest is completed
								local entries, _quests = GetNumQuestLogEntries()
								for z=1,entries do
									local title, _level, _tag, _group, header, _collapsed, complete, _daily = GetQuestLogTitle(z)
									if (title and not header and complete) then
										if string.find(string.lower(FIZ_FG_fs_h_d.name), string.lower(title)) then
											-- this quest matches
											result = result + FIZ_FG_fs_h_d.rep
										end
									end
								end
							end
						end
					end
				end
--[[ TODO: Fix this!
				-- items
				local FIZ_FG_fs_h=FIZ_FG_fs.items
				if (FIZ_FG_fs_h and MFAC_Data.ShowItems) then
					for i = 0, FIZ_FG_fs_h.count do
					local FIZ_FG_fs_h_d=FIZ_FG_fs_h.data[i]
						if (not FIZ_FG_fs_h_d.limit or (faction.bar_value < FIZ_FG_fs_h_d.limit)) then
							local toDo = ceil(repToNext / FIZ_FG_fs_h_d.rep)
							if (FIZ_FG_fs_h_d.limit) then
								toDo = ceil((FIZ_FG_fs_h_d.limit - faction.bar_value) / FIZ_FG_fs_h_d.rep)
							end
							if (FIZ_FG_fs_h_d.items) then
								local currentQuestTimesBag = -1
								local currentQuestTimesBagBank = -1
								for item in pairs(FIZ_FG_fs_h_d.items) do
									if (FIZ_ItemsCarried and FIZ_ItemsCarried[item]) then
										if ((FIZ_ItemsCarried[item] >= FIZ_FG_fs_h_d.items[item]) and (FIZ_FG_fs_h_d.items[item] > 0)) then
											local localCurrentTimesBag = floor(FIZ_ItemsCarried[item] / FIZ_FG_fs_h_d.items[item])
											if (currentQuestTimesBag == -1) then
												-- first items for this quest --> take value
												currentQuestTimesBag = localCurrentTimesBag
											else
												-- some items already Set
												if (localCurrentTimesBag < currentQuestTimesBag) then
													-- fewer of this item than of others, reduce quest count
													currentQuestTimesBag = localCurrentTimesBag
												end
											end
										else
											-- not enough of this item for quest -> set to 0
											currentQuestTimesBag = 0
										end
										if (MFAC_Data.Bank[MFAC_Data.player_bank_key][item]) then
											local total = MFAC_Data.Bank[MFAC_Data.player_bank_key][item] + FIZ_ItemsCarried[item]
											if ((total >= FIZ_FG_fs_h_d.items[item]) and (FIZ_FG_fs_h_d.items[item] > 0)) then
												local localCurrentTimesBagBank = floor(total / FIZ_FG_fs_h_d.items[item])
												if (currentQuestTimesBagBank == -1) then
													-- first items for this quest --> take value
													currentQuestTimesBagBank = localCurrentTimesBagBank
												else
													-- some items already Set
													if (localCurrentTimesBagBank < currentQuestTimesBagBank) then
														-- fewer of this item than of others, reduce quest count
														currentQuestTimesBagBank = localCurrentTimesBagBank
													end
												end
											else
												-- not enough of this item for quest -> set to 0
												currentQuestTimesBagBank = 0
												FIZ_UpdateList[itemIndex].name = FIZ_UpdateList[itemIndex].name.." ["..total.."x]"
											end
										else
											-- none of this carried In bank
										end
									else
										-- not enough of this item for quest -> set to 0
										currentQuestTimesBag = 0
									end
								end
								if (currentQuestTimesBag > toDo) then
									currentQuestTimesBag = toDo
								end
								if (currentQuestTimesBagBank > toDo) then
									currentQuestTimesBagBank = toDo
								end
								if (currentQuestTimesBagBank > 0) then
									result = result + currentQuestTimesBagBank * FIZ_FG_fs_h_d.rep
								elseif (currentQuestTimesBag > 0) then
									result = result + currentQuestTimesBag * FIZ_FG_fs_h_d.rep
								end
							end
						end
					end
				end
				--]]
			end
		end
	end

	return result;
end


function MFAC.UpdateOriginalReputationDetailFrame(faction, idx, i)

	ReputationDetailFactionName:SetText(faction.name);
	ReputationDetailFactionDescription:SetText(faction.description);

	if ( faction.at_war ) then
		ReputationDetailAtWarCheckBox:SetChecked(1);
	else
		ReputationDetailAtWarCheckBox:SetChecked(nil);
	end

	if (faction.can_toggle_at_war) then
		ReputationDetailAtWarCheckBox:Enable();
		ReputationDetailAtWarCheckBoxText:SetTextColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
	else
		ReputationDetailAtWarCheckBox:Disable();
		ReputationDetailAtWarCheckBoxText:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
	end

	-- When Sorting by Standing, "Inactive" factions aren't supported
	if (not MFAC_Data.RepGroupBy == MFAC.RepGroupByOptions.Standing) then
		if ( not faction.is_header ) then
			ReputationDetailInactiveCheckBox:Enable();
			ReputationDetailInactiveCheckBoxText:SetTextColor(ReputationDetailInactiveCheckBoxText:GetFontObject():GetTextColor());
		else
			ReputationDetailInactiveCheckBox:Disable();
			ReputationDetailInactiveCheckBoxText:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
		end
		MFAC.ShowReputationBarHighlight(i)
	end

	if ( IsFactionInactive(idx) ) then
		ReputationDetailInactiveCheckBox:SetChecked(1);
	else
		ReputationDetailInactiveCheckBox:SetChecked(nil);
	end

	if ( faction.is_watched ) then
		ReputationDetailMainScreenCheckBox:SetChecked(1);
	else
		ReputationDetailMainScreenCheckBox:SetChecked(nil);
	end
end

function FIZ_Rep_Detail_Frame(p_faction_idx, p_faction_info, p_color, barValue, barMax, toExalted, factionStandingtext)

	FIZ_BuildUpdateList(p_faction_info.faction.faction_id)

	local name = p_faction_info.faction.name
	local atWarWith = p_faction_info.faction.at_war
	local canToggleAtWar = p_faction_info.faction.can_toggle_at_war
	local isWatched = p_faction_info.faction.is_watched
	UI.detail_faction_name:SetText(name);
	UI.detail_faction_desc:SetText(p_faction_info.faction.description);


	UI.detail_current_standing_value:SetText(factionStandingtext)
	UI.detail_current_standing_value:SetTextColor(p_color.r, p_color.g, p_color.b)
	UI.detail_current_reputation_value:SetText(FormatLargeNumber(barValue))
	UI.detail_reputation_needed_value:SetText(FormatLargeNumber(barMax))
	UI.detail_reputation_missing_value:SetText(FormatLargeNumber(barMax - barValue))
	UI.detail_reputation_in_bag_value:SetText(FormatLargeNumber(FIZ_CurrentRepInBag))
	UI.detail_reputation_in_bag_bank_value:SetText(FormatLargeNumber(FIZ_CurrentRepInBagBank))
	UI.detail_reputation_in_quests_value:SetText(FormatLargeNumber(FIZ_CurrentRepInQuest))

	if (MFAC.StoredRep and MFAC.StoredRep[name] and MFAC.StoredRep[name].session_gain) then
		UI.detail_reputation_session_value:SetText(FormatLargeNumber(MFAC.StoredRep[name].session_gain))
	else
		UI.detail_reputation_session_value:SetText("")
	end

	UI.detail_reputation_to_exalted_value:SetText(FormatLargeNumber(toExalted))

	if ( atWarWith ) then
		UI.detail_at_war_checkbox:SetChecked(1)
	else
		UI.detail_at_war_checkbox:SetChecked(false)
	end

	if (canToggleAtWar) then
		UI.detail_at_war_checkbox:Enable();
		UI.detail_at_war_checkbox.fnt_string:SetTextColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
	else
		UI.detail_at_war_checkbox:Disable();
		UI.detail_at_war_checkbox.fnt_string:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
	end

	if ( IsFactionInactive(p_faction_idx) ) then
		UI.detail_move_to_inactive_checkbox:SetChecked(1);
	else
		UI.detail_move_to_inactive_checkbox:SetChecked(nil);
	end

	if ( isWatched ) then
		UI.detail_watch_faction_checkbox:SetChecked(1);
	else
		UI.detail_watch_faction_checkbox:SetChecked(nil);
	end
end

-----------------------------------
-- _16_ Listing by standing
-----------------------------------
function MFAC.PrintByStanding(p_standing_id)
	local _, faction_data = MFAC.GetFactionDataByStanding()
	local sex = UnitSex("player")

	for _idx, v in ipairs(faction_data) do
		local standing_id = v.faction.standing_id;
		if (p_standing_id and standing_id == p_standing_id) or (not p_standing_id) then
			if (v.faction.is_header) then
				local factionStandingtext, colorIndex = MFAC.get_faction_standing(v, sex)
				local colour = MFAC.RGBColourString(1, FACTION_BAR_COLORS[colorIndex].r, FACTION_BAR_COLORS[colorIndex].g, FACTION_BAR_COLORS[colorIndex].b);
				MFAC.Print("|n" .. colour .. "--- " .. factionStandingtext .. " (" .. standing_id .. ") ---|r")
			else
				MFAC.Print("    ".. v.faction.name .. ": " .. FIZ_TXT.missing2 .. ": " .. v.faction.bar_max - v.faction.bar_value)
			end
		end
	end
end


--------------------------
-- _22_ Rep Main Window
--------------------------



function MuffinFactionizerAlertFrame_OnClick(self, ...) --luacheck: ignore 111 212
	if (self.delay == -1) then
		self:SetScript("OnLeave", AlertFrame_ResumeOutAnimation)
		self.delay = 0
	end
		self.animIn:Stop();
		if self.glow then
		self.glow.animIn:Stop();
		end
		if self.shine then
		self.shine.animIn:Stop();
		end
		self.waitAndAnimOut:Stop();
		self:Hide();
		return true;
end

function MFAC.AlertFrame_SetUp(frame, achievementID, name, delay, toptext, onClick, icon)
	-- An alert flagged as alreadyEarned has more space for the text to display since there's no shield+points icon.
	AchievementAlertFrame_SetUp(frame, achievementID, true)
	frame.Name:SetText(name)
	frame.Unlocked:SetText(toptext or ACHIEVEMENT_UNLOCKED)
	frame.onClick = onClick
	frame.delay = delay

	if (delay) then
		if (delay <= 0) then
			C_Timer.After(0, function()  AlertFrame_PauseOutAnimation(frame);  end)
		else
			C_Timer.After(0, function() frame.waitAndAnimOut.animOut:SetStartDelay(delay) end)
		end
	end
	if (delay == -1) then
		frame:SetScript("OnLeave", nil)
	else
		frame:SetScript("OnLeave", AlertFrame_ResumeOutAnimation)
	end
	if (icon) then
		frame.Icon.Texture:SetTexture(icon)
		frame.Icon.Texture:SetTexCoord(0, 0, 0, 1, 1, 0, 1, 1)
		frame.Background:SetTexture("Interface\\AchievementFrame\\UI-Achievement-Alert-Background")
		frame.OldAchievement:SetTexture("Interface\\AchievementFrame\\UI-Achievement-Borders")
	else
		frame.Icon.Texture:SetTexCoord(0, 0, 0, 1, 1, 0, 1, 1)
		frame.Background:SetTexture("Interface\\AchievementFrame\\UI-Achievement-Alert-Background")
		frame.OldAchievement:SetTexture("Interface\\AchievementFrame\\UI-Achievement-Borders")
	end
end


function MFAC.ToastFakeAchievement(name, p_icon, delay, toptext, onClick)

	--Make sure Blizzard's Achievement frame has been loaded
	if ( not AchievementFrame ) then
		AchievementFrame_LoadUI();
		--print("     not AchievementFrame" );
	end

	if (not MFAC.AlertSystem) then
		MFAC.AlertSystem = AlertFrame:AddQueuedAlertFrameSubSystem("MuffinFactionizerAlertFrameTemplate", MFAC.AlertFrame_SetUp, 4, math.huge)
	end

  local baseID = 519;-- 519 is Rep hands shaking
  MFAC.AlertSystem:AddAlert(baseID, name, delay, toptext, onClick, p_icon)

  --if (playSound) then  PlaySound(12891);  end
  --if (chatMessage) then  chatprint("", chatMessage);  end
end


--MFAC_TEST_ToastFakeAchievement = MFAC.ToastFakeAchievement;
--	/run	MFAC_TEST_ToastFakeAchievement("Dreamweavers!", nil, -1, "Paragon Reward Earned", nil, 1394953)

function MFAC.is_quest_ready_for_turnin(p_quest_id)
	local objectives = C_QuestLog.GetQuestObjectives(p_quest_id)

	if(objectives == nil) then
		print("MuffinFactionizer Warning: Quest", p_quest_id, "seems to have no objectives")
		return false;
	end

	for _, info in pairs(objectives) do
		if(info.finished ~= true) then
			return false
		end
	end

	return true
end

function MFAC.check_for_pending_rewards()

	if (WOW_PROJECT_ID == WOW_PROJECT_CLASSIC) then
		return
	end

	for _quest_id, faction_id in pairs(MuffinFactionizer.paragon_supply_quests) do
			local reward_pending = select(4, MFAC.GetFactionParagonInfo(faction_id));

		if(reward_pending ) then
			local faction_name = GetFactionInfoByID(faction_id);
			local icon_id = MuffinFactionizer.faction_toast_icons[faction_id]
			MFAC.ToastFakeAchievement(faction_name, icon_id, -1, "Paragon Reward Available");
		end
	end

	for quest_id, faction_id in pairs(MuffinFactionizer.emissary_reward_quests) do
		if(C_QuestLog.IsOnQuest(quest_id) and MFAC.is_quest_ready_for_turnin(quest_id)) then
			local faction_name = GetFactionInfoByID(faction_id);
			local icon_id = MuffinFactionizer.faction_toast_icons[faction_id]
			MFAC.ToastFakeAchievement(faction_name, icon_id, -1, "Emissary Reward Available");
		end
	end

end

function MuffinFactionizer.GetMFAC()
	return MFAC;
end

function MFAC.AddMapping(p_name, p_id)
	MFAC.FactionIDToName[p_id] = p_name;
	MFAC.FactionNameToID[p_name] = p_id;
end

function MFAC.InitEnFactions()
local numFactions = GetNumFactions()
	for i=1, numFactions do
		local name, _, _, _, _, _, _, _, is_header, _, _, _, _, faction_id = GetFactionInfo(i)
		if (name and faction_id and not is_header) then
			MFAC.AddMapping(name, faction_id)
		end
	end
end

function MFAC.add_faction_info_to_tooltip(p_faction_id, p_tooltip)
	local name, _, standingID, _barMin, barMax, barValue = GetFactionInfoByID(p_faction_id);
	local msg = "|n" .. format(FACTION_STANDING_CHANGED, _G["FACTION_STANDING_LABEL" .. standingID], name);

	if(standingID < MuffinFactionizer.standing.Exalted) then
      msg = msg .. " (" .. tostring(barMax - barValue) .. ")"
	end
			--TODO: Handle Paragon standing

	p_tooltip:AddLine(msg, MuffinFactionizer.TooltipStandingFontColour.r, MuffinFactionizer.TooltipStandingFontColour.g, MuffinFactionizer.TooltipStandingFontColour.b);

end


function MuffinFactionizer.AddItemLines(p_tooltip, p_link)
	if (not p_link) then return end
	if (not p_tooltip) then return end
   local item_id = tonumber(string.match(p_link,"item:(%d+)"))
	if (not item_id) then return end

	local item_info = MuffinFactionizer.ItemToFactionMap[item_id]

	if(item_info) then
		for k in pairs(item_info) do
			MFAC.add_faction_info_to_tooltip(k, p_tooltip)
		end
		p_tooltip:Show()
	end

end

function MuffinFactionizer.AddCurrencyLines(p_tooltip, p_currency_id)
	if (not p_currency_id) then return end
	if (not p_tooltip) then return end

	local currency_info = MuffinFactionizer.CurrencyToFactionMap[p_currency_id]

	if(currency_info) then
		for k in pairs(currency_info) do
			MFAC.add_faction_info_to_tooltip(k, p_tooltip)
		end
		p_tooltip:Show()
	end

end


local function OnTooltipSetItem(tooltip, ...) --luacheck: ignore 212/...
   local _name, link = tooltip:GetItem();
	MuffinFactionizer.AddItemLines(tooltip, link)
end

local function SetCurrencyByID(p_tooltip, p_currency_id)
	MuffinFactionizer.AddCurrencyLines(p_tooltip, p_currency_id)
end



local function onSetHyperlink(tooltip, link)
	MuffinFactionizer.AddItemLines(tooltip, link)
end

--This is copied from WorldMapBountyBoard.lua:303
local function AddObjectives(questID, numObjectives)
	for objectiveIndex = 1, numObjectives do
		local objectiveText, _objectiveType, finished = GetQuestObjectiveInfo(questID, objectiveIndex, false);
		if objectiveText and #objectiveText > 0 then
			local color = finished and GRAY_FONT_COLOR or HIGHLIGHT_FONT_COLOR;
			WorldMapTooltip:AddLine(QUEST_DASH .. objectiveText, color.r, color.g, color.b, true);
		end
	end
end

local function ShowBountyTooltip(_self, bountyIndex)
	local bountyData = MFAC.bounty_frame.bounties[bountyIndex];
	local questIndex = GetQuestLogIndexByID(bountyData.questID);
	local title, _level, _suggestedGroup, _isHeader, _isCollapsed, _isComplete, _frequency, questID, _startEvent, _displayQuestID, _isOnMap, _hasLocalPOI, _isTask, _isStory = GetQuestLogTitle(questIndex);
	local quest_faction = MuffinFactionizer.emissary_reward_quests[questID]
	if(not quest_faction) then
		return; -- This isn't supported (yet) so bail out TODO: output some debug stuff here
	end

	if(not title) then
		return;	-- Invalid quest?
	end

	WorldMapTooltip:ClearLines()

	WorldMapTooltip:SetText(title, HIGHLIGHT_FONT_COLOR:GetRGB());
	WorldMap_AddQuestTimeToTooltip(bountyData.questID);

	local _, questDescription = GetQuestLogQuestText(questIndex);
	WorldMapTooltip:AddLine(questDescription, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, true);
	AddObjectives(bountyData.questID, bountyData.numObjectives);

	MFAC.add_faction_info_to_tooltip(quest_faction, WorldMapTooltip)

	if bountyData.turninRequirementText then
		WorldMapTooltip:AddLine(bountyData.turninRequirementText, RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b, true);
	end

	GameTooltip_AddQuestRewardsToTooltip(WorldMapTooltip, bountyData.questID);

	WorldMapTooltip:Show();

end

function MFAC.set_method_hooks()

	hooksecurefunc(ItemRefTooltip, "SetHyperlink", onSetHyperlink)
	GameTooltip:HookScript("OnTooltipSetItem", OnTooltipSetItem)

	if (WOW_PROJECT_ID ~= WOW_PROJECT_CLASSIC) then
		hooksecurefunc(GameTooltip, "SetCurrencyByID", SetCurrencyByID)

		for _,frame in ipairs(WorldMapFrame.overlayFrames) do
		   if frame.ShowBountyTooltip then
		      hooksecurefunc(frame, "ShowBountyTooltip", ShowBountyTooltip)
				MFAC.bounty_frame = frame
		   end
		end

	end

	hooksecurefunc("ReputationBar_OnClick", MFAC.ReputationBar_OnClick)

end

MFAC.MainWindow = CreateFrame("FRAME", "MuffinFactionizerEventFrame")
MFAC.MainWindow:RegisterEvent("PLAYER_ENTERING_WORLD")
MFAC.MainWindow:SetScript("OnEvent", function(self, event, ...)
	if(self[event]) then
		self[event](self, ...);
	else
		FIZ_OnEvent(self, event, ...);
	end

end);



function MFAC.MainWindow.PLAYER_ENTERING_WORLD(_self)
	FIZ_Init()

	MFAC.MainWindow:UnregisterEvent("PLAYER_ENTERING_WORLD")
	MFAC.MainWindow:RegisterEvent("UPDATE_FACTION")

	if (WOW_PROJECT_ID ~= WOW_PROJECT_CLASSIC) then
		MFAC.MainWindow:RegisterEvent("AZERITE_ITEM_EXPERIENCE_CHANGED")
	end
	-- to keep item list up to date
	MFAC.MainWindow:RegisterEvent("BAG_UPDATE_DELAYED")
	MFAC.MainWindow:RegisterEvent("BANKFRAME_OPENED")
	MFAC.MainWindow:RegisterEvent("BANKFRAME_CLOSED")

	-- to keep list of known skills up to date
	MFAC.MainWindow:RegisterEvent("CHAT_MSG_SKILL")
	MFAC.MainWindow:RegisterEvent("TRADE_SKILL_LIST_UPDATE")
	MFAC.MainWindow:RegisterEvent("SKILL_LINES_CHANGED")
	MFAC.MainWindow:RegisterEvent("UPDATE_TRADESKILL_RECAST")

	MFAC.MainWindow:RegisterEvent("QUEST_TURNED_IN")
	MFAC.MainWindow:RegisterEvent("QUEST_WATCH_UPDATE")
	MFAC.MainWindow:RegisterEvent("QUEST_ACCEPTED")
	MFAC.MainWindow:RegisterEvent("QUEST_LOG_UPDATE")

	MFAC.MainWindow:RegisterEvent("GOSSIP_SHOW") --Testing auto-turnin stuff
	MFAC.MainWindow:RegisterEvent("QUEST_PROGRESS") --Testing auto-turnin stuff
	MFAC.MainWindow:RegisterEvent("QUEST_FINISHED") --Testing auto-turnin stuff
	MFAC.MainWindow:RegisterEvent("QUEST_COMPLETE") --Testing auto-turnin stuff


	MFAC.MainWindow:RegisterEvent("PLAYER_REGEN_ENABLED") --Track end of combat so we don't taint stuff

	-- chat hook system
	ChatFrame_AddMessageEventFilter("CHAT_MSG_COMBAT_FACTION_CHANGE", FIZ_ChatFilter)
	ChatFrame_AddMessageEventFilter("COMBAT_TEXT_UPDATE", FIZ_ChatFilter)

	MFAC.set_method_hooks();

	MFAC.show_whats_new();

	if (MFAC_Data.ShowToasts) then
	  C_Timer.After(0, function()
	    -- This strange double-timer thing works around an issue where the timer starts counting down, so to speak, during the loading screen if the UI is being reloaded (as per /reload), making the toast not appear.
	    C_Timer.After(4, MFAC.check_for_pending_rewards)
	  end)
	end


end





-- Slash commands for CLI
SLASH_FIZ1 = "/muffinfactionizer"
SLASH_FIZ2 = "/mfac"
SLASH_FIZ3 = "/mfz"
SlashCmdList.FIZ = FIZ_SlashHandler

FIZ_Orig_ReputationFrame_Update = ReputationFrame_Update
ReputationFrame_Update = MFAC.ReputationFrame_Update 	-- luacheck: ignore 121 (setting read-only variable)

MFAC.Orig_ExpandFactionHeader = ExpandFactionHeader
ExpandFactionHeader = MFAC.ExpandFactionHeader	-- luacheck: ignore 121 (setting read-only variable)

MFAC.Orig_CollapseFactionHeader = CollapseFactionHeader
CollapseFactionHeader = MFAC.CollapseFactionHeader	-- luacheck: ignore 121 (setting read-only variable)

MFAC.original_standing_text = ReputationFrameStandingLabel:GetText()
