local ADDON_NAME, MFAC = ... -- Pulls back the Addon-Local Variables and store them locally.
local UI = MFAC.UI
local UITK = MFAC.UITK

local detail_frame_coords = {}

if (WOW_PROJECT_ID == WOW_PROJECT_CLASSIC) then
	detail_frame_coords.x = -20
	detail_frame_coords.y = 0
else
	detail_frame_coords.x = 2
	detail_frame_coords.y = 0
end

function MFAC.build_frames()

	MFAC.build_rep_frame_widgets()
	MFAC.build_detail_frame();
	MFAC.build_options_frame();

end

function MFAC.build_detail_frame()
	UI.ReputationDetailFrame = CreateFrame("FRAME", "MFAC_ReputationDetailFrame", ReputationFrame, BackdropTemplateMixin and "BackdropTemplate")
	local rdf = UI.ReputationDetailFrame
	rdf:EnableMouse(true)
	rdf:EnableMouseWheel(true)
	rdf:SetBackdrop({
		bgFile = "Interface/ChatFrame/ChatFrameBackground",
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true,
		tileSize = 32,
		edgeSize = 16,
		insets = { left = 6, right = 6, top = 4, bottom = 4 }
	})
	rdf:SetBackdropColor(0, 0, 0, 0.97);
	rdf:SetPoint("TOPLEFT", ReputationFrame, "TOPRIGHT", detail_frame_coords.x, detail_frame_coords.y)
	rdf:SetSize(450, 570);
	rdf:SetScript("OnMouseWheel", function(_self, p_delta)
		local s_min, s_max = UI.detail_slider:GetMinMaxValues()
		local val = UI.detail_slider:GetValue()
		local new_value = Clamp(val - (p_delta * 3), s_min, s_max)
		UI.detail_slider:SetValue(new_value)
end)
	rdf:Hide();


	---------------
	-- Top bar
	---------------
	UI.detail_faction_name = UITK.create_fontstring(rdf, nil)
	UI.detail_faction_name:SetTextColor(1, 0.82, 0)
	UI.detail_faction_name:SetFont(UITK.default_font_name, 13, "OUTLINE")
	UI.detail_faction_name:SetPoint("TOP", rdf, "TOP", 0, -8)

	UI.detail_close_button = CreateFrame("Button", nil, rdf, "UIPanelCloseButton")
	UI.detail_close_button:SetPoint("TOPRIGHT", rdf, "TOPRIGHT", 3, 2)
	UI.detail_close_button:SetScript("OnClick", function(self, _button, _down)
		UITK.PlayClick(true)
		HideParentPanel(self);
	end)


	UI.detail_title_divider = UITK.create_divider(rdf, nil);
	UI.detail_title_divider:SetPoint("TOPLEFT", 5, -19)
	UI.detail_title_divider:SetPoint("RIGHT", rdf, "RIGHT", -4, 0)

	UI.detail_faction_overview_texture = UITK.CreateTexture(rdf, "Interface/PaperDollInfoFrame/UI-Character-Reputation-DetailBackground", 0, 132);
	UI.detail_faction_overview_texture:SetPoint("TOPLEFT", UI.detail_title_divider, "TOPLEFT", 0, -8)
	UI.detail_faction_overview_texture:SetPoint("RIGHT", UI.detail_title_divider, "CENTER")
	UI.detail_faction_overview_texture:SetTexCoord(0.02, 0.715, 0.01, 0.95)
	UI.detail_faction_overview_texture:SetVertexColor(0.7, 0.7, 0.7, 1)

	UI.detail_faction_desc = UITK.create_fontstring(rdf, nil)
	UI.detail_faction_desc:SetFont(UITK.default_font_name, 13)
	UI.detail_faction_desc:SetPoint("TOPLEFT", UI.detail_faction_overview_texture, "TOPLEFT", 8, -6)
	UI.detail_faction_desc:SetPoint("BOTTOMRIGHT", UI.detail_faction_overview_texture, "BOTTOMRIGHT", -3, 5)
	UITK.make_fontstring_resizable(UI.detail_faction_desc, 7, 13)

	--
	-- Faction Detail section (TopRight)
	--
	UI.detail_faction_detail_texture = UITK.CreateTexture(rdf, "Interface/PaperDollInfoFrame/UI-Character-Reputation-DetailBackground");
	UI.detail_faction_detail_texture:SetPoint("TOPLEFT", UI.detail_faction_overview_texture, "TOPRIGHT")
	UI.detail_faction_detail_texture:SetPoint("BOTTOMLEFT", UI.detail_faction_overview_texture, "BOTTOMRIGHT")
	UI.detail_faction_detail_texture:SetPoint("RIGHT", rdf, "RIGHT", -6, 0)
	UI.detail_faction_detail_texture:SetTexCoord(0.02, 0.715, 0.01, 0.95)
	UI.detail_faction_detail_texture:SetVertexColor(0.7, 0.7, 0.7, 1)

	local line_spacing = -4
	local strings1 ={
		{name = "detail_current_standing_label",			text = MFAC.TXT.current_standing, relative_to = UI.detail_faction_detail_texture, anchor2 = "TOPLEFT", x = 8, y = -6},
		{name = "detail_current_reputation_label", 		text = FIZ_TXT.currentRep},
		{name = "detail_reputation_needed_label", 		text = FIZ_TXT.neededRep},
		{name = "detail_reputation_missing_name", 		text = FIZ_TXT.missingRep},
		{name = "detail_reputation_in_bag_label", 		text = FIZ_TXT.repInBag},
		{name = "detail_reputation_in_bag_bank_label",	text = FIZ_TXT.repInBagBank},
		{name = "detail_reputation_in_quests_label",		text = FIZ_TXT.repInQuest},
		{name = "detail_reputation_session_label",		text = FIZ_TXT.factionGained},
		{name = "detail_reputation_to_exalted_label",	text = FIZ_TXT.toExalted},

		{name = "detail_current_standing_value",		anchor1 = "TOPRIGHT", anchor2 = "TOPLEFT", relative_to = "detail_current_standing_label", x = 210, y = 0},
		{name = "detail_current_reputation_value",	anchor1 = "TOPRIGHT", anchor2 = "BOTTOMRIGHT"},
		{name = "detail_reputation_needed_value",		anchor1 = "TOPRIGHT", anchor2 = "BOTTOMRIGHT"},
		{name = "detail_reputation_missing_value",	anchor1 = "TOPRIGHT", anchor2 = "BOTTOMRIGHT"},
		{name = "detail_reputation_in_bag_value",		anchor1 = "TOPRIGHT", anchor2 = "BOTTOMRIGHT"},
		{name = "detail_reputation_in_bag_bank_value", anchor1 = "TOPRIGHT", anchor2 = "BOTTOMRIGHT"},
		{name = "detail_reputation_in_quests_value",	anchor1 = "TOPRIGHT", anchor2 = "BOTTOMRIGHT"},
		{name = "detail_reputation_session_value", anchor1 = "TOPRIGHT", anchor2 = "BOTTOMRIGHT"},
		{name = "detail_reputation_to_exalted_value", anchor1 = "TOPRIGHT", anchor2 = "BOTTOMRIGHT"},
	};

	for i, v in ipairs(strings1) do
		local fnt_string = UITK.create_fontstring(rdf, nil)
		UI[v.name] = fnt_string;
		local x, y = (v.x or 0), (v.y or line_spacing)
		local relative_to = v.relative_to or strings1[i-1].name
		if(type(v.relative_to) == "string" or v.relative_to == nil) then
			relative_to = UI[v.relative_to or strings1[i-1].name];
		elseif(type(v.relative_to) == "table") then
			relative_to = v.relative_to
		end

		fnt_string:SetSize(0, 10)
		fnt_string:SetPoint(v.anchor1 or "TOPLEFT", relative_to, v.anchor2 or "BOTTOMLEFT", x, y)
		fnt_string:SetText(v.text)
	end

	--
	-- Faction Detail Checkbox section
	--
	UI.detail_checkbox_divider = UITK.create_divider(rdf, nil);
	UI.detail_checkbox_divider:SetPoint("TOPLEFT", UI.detail_faction_overview_texture, "BOTTOMLEFT", 0, 4)
	UI.detail_checkbox_divider:SetPoint("RIGHT", UI.detail_title_divider, "RIGHT", 0, 0)

	UI.detail_at_war_checkbox = UITK.create_checkbox(rdf, nil, AT_WAR, {tip_title = AT_WAR, tip = REPUTATION_AT_WAR_DESCRIPTION} )
	UI.detail_at_war_checkbox:SetPoint("TOPLEFT", UI.detail_checkbox_divider, "BOTTOMLEFT", 5, 2)
	UI.detail_at_war_checkbox:SetCheckedTexture("Interface/Buttons/UI-CheckBox-SwordCheck")
	UI.detail_at_war_checkbox:GetCheckedTexture():ClearAllPoints()
	UI.detail_at_war_checkbox:GetCheckedTexture():SetPoint("CENTER", 0, -1)
	UI.detail_at_war_checkbox:GetCheckedTexture():SetSize(24, 24)
	UI.detail_at_war_checkbox:GetCheckedTexture():SetTexCoord(0, 0.6, 0, 0.6)
	UI.detail_at_war_checkbox:SetScript("OnClick", function(self)
		FactionToggleAtWar(GetSelectedFaction());
		UITK.PlayClick(self:GetChecked())
		ReputationFrame_Update();
	end)

	UI.detail_move_to_inactive_checkbox = UITK.create_checkbox(rdf, nil, MOVE_TO_INACTIVE, {tip_title = MOVE_TO_INACTIVE, tip = REPUTATION_MOVE_TO_INACTIVE})
	UI.detail_move_to_inactive_checkbox.fnt_string:SetTextColor(1, 0.82, 0)
	UI.detail_move_to_inactive_checkbox:SetPoint("TOPLEFT", UI.detail_at_war_checkbox, "TOPRIGHT", 60, 0)
	UI.detail_move_to_inactive_checkbox:SetScript("OnClick", function(self)
		if ( self:GetChecked() ) then
			PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON);
			SetFactionInactive(GetSelectedFaction());
		else
			PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_OFF);
			SetFactionActive(GetSelectedFaction());
		end
	end)

	UI.detail_watch_faction_checkbox = UITK.create_checkbox(rdf, nil, SHOW_FACTION_ON_MAINSCREEN, {tip_title = SHOW_FACTION_ON_MAINSCREEN, tip = REPUTATION_SHOW_AS_XP} )
	UI.detail_watch_faction_checkbox.fnt_string:SetTextColor(1, 0.82, 0)
	UI.detail_watch_faction_checkbox:SetPoint("TOPLEFT", UI.detail_move_to_inactive_checkbox, "TOPRIGHT", 125, 0)
	UI.detail_watch_faction_checkbox:SetScript("OnClick", function(self)
		if ( self:GetChecked() ) then
			PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON);
			SetWatchedFactionIndex(GetSelectedFaction());
		else
			PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_OFF);
			SetWatchedFactionIndex(0);
		end
		ReputationFrame_Update();
	end)


	UI.detail_objectives_divider = UITK.create_divider(rdf, nil);
	UI.detail_objectives_divider:SetPoint("TOPLEFT", UI.detail_checkbox_divider, "BOTTOMLEFT", 0, -20)
	UI.detail_objectives_divider:SetPoint("RIGHT", UI.detail_checkbox_divider, "RIGHT", 0, 0)

	UI.detail_slider = CreateFrame("Slider", nil, rdf, BackdropTemplateMixin and "BackdropTemplate");

	UI.detail_slider_button_scroll_up = CreateFrame("Button", nil, rdf, "UIPanelScrollUpButtonTemplate");
	UI.detail_slider_button_scroll_up:SetPoint("TOPRIGHT", UI.detail_objectives_divider, "BOTTOMRIGHT", -3, 0)
	UI.detail_slider_button_scroll_up.MFAC_slider = UI.detail_slider
	UI.detail_slider_button_scroll_up:SetScript("OnClick", function(self)
		local slider = self.MFAC_slider;
		local slide_min, _slide_max = slider:GetMinMaxValues()
		local current = slider:GetValue()
		if (current > slide_min) then slider:SetValue(current - 1); PlaySound(SOUNDKIT.U_CHAT_SCROLL_BUTTON); end;
		if ((current - 1) <= slide_min) then self:Disable(); end;
end)
	UI.detail_no_info_text = UITK.create_fontstring(rdf, nil)
	UI.detail_no_info_text:SetSize(0, 90)
	UI.detail_no_info_text:SetPoint("TOPLEFT", UI.detail_objectives_divider, "BOTTOMLEFT", 10, -10)
	UI.detail_no_info_text:SetPoint("RIGHT", UI.detail_objectives_divider, "RIGHT", -10, 0)
	UITK.make_fontstring_resizable(UI.detail_no_info_text, 7, 13)
	UI.detail_no_info_text:SetText(FIZ_TXT.noInfo)


	UI.detail_slider:SetBackdrop({
		bgFile = "Interface/Buttons/UI-SliderBar-Background",
		tile = true,
		tileSize = 8,
	})
	UI.detail_slider:SetBackdropColor(0, 0, 0, 0.97);
	UI.detail_slider:SetThumbTexture("Interface/Buttons/UI-ScrollBar-Knob")
	UI.detail_slider:GetThumbTexture():SetTexCoord(0.20, 0.80, 0.125, 0.875)
	UI.detail_slider:GetThumbTexture():SetSize(18, 26)
	UI.detail_slider:SetSize(20, 208);
	UI.detail_slider.original_width = 20;
	UI.detail_slider:SetMinMaxValues(1, 13)
	UI.detail_slider:SetValue(1)
	UI.detail_slider:SetValueStep(1)
	UI.detail_slider:SetPoint("TOPRIGHT", UI.detail_slider_button_scroll_up, "BOTTOMRIGHT", 1, 2)
	UI.detail_slider:SetOrientation('VERTICAL')
	UI.detail_slider:SetObeyStepOnDrag(true)
	UI.detail_slider:SetScript("OnValueChanged", function (_self, value)
		MFAC.UpdateDetailList(value)
end)


	UI.detail_slider_button_scroll_down = CreateFrame("Button", nil, rdf, "UIPanelScrollDownButtonTemplate");
	UI.detail_slider_button_scroll_down.MFAC_slider = UI.detail_slider
	UI.detail_slider_button_scroll_down:SetPoint("TOPRIGHT", UI.detail_slider, "BOTTOMRIGHT", -1, 2)
	UI.detail_slider_button_scroll_down:SetScript("OnClick", function(self)
		local slider = self.MFAC_slider;
		local _slide_min, slide_max = slider:GetMinMaxValues()
		local current = slider:GetValue()
		if (current < slide_max) then slider:SetValue(current + 1); PlaySound(SOUNDKIT.U_CHAT_SCROLL_BUTTON); end;
		if ((current + 1) >= slide_max) then self:Disable(); end;
end)

	UI.detail_entry_buttons = {}
	for i = 1, 13 do
		local b = CreateFrame("Button", nil, rdf, BackdropTemplateMixin and "BackdropTemplate");
		b:SetSize(0, 16)
		if(i == 1) then
			b:SetPoint("TOPLEFT", UI.detail_objectives_divider, "BOTTOMLEFT", 5, -2)
		else
			b:SetPoint("TOPLEFT", UI.detail_entry_buttons[i-1], "BOTTOMLEFT", 0, -2)
		end
		b:SetPoint("RIGHT", UI.detail_slider, "LEFT", 0, 0)
		b:SetBackdrop({bgFile = "Interface/ChatFrame/ChatFrameBackground", tile = true, tileSize = 8})
	   if(i%2 == 0) then
	      b:SetBackdropColor(1,0.3,1, 0.1)
	   else
	      b:SetBackdropColor(0.3,0.3,1, 0.1)
	   end

		b:RegisterForClicks("LeftButtonUp", "RightButtonUp")
		b:SetScript("OnEnter", function (self) FIZ_ShowLineToolTip(self); end)
		b:SetScript("OnLeave", function (_self) GameTooltip:Hide(); end)
		b:SetScript("OnMouseUp", function(self, button) FIZ_MouseButtonUp(self, button); end)


		b.mfac_texture = UITK.CreateTexture(b, "Interface/Addons/" .. ADDON_NAME .. "/Textures/UI-MinusButton-UP-Green.tga", 16, 16);
		b.mfac_texture:SetPoint("LEFT", b, "LEFT", 3, 0)

		b.mfac_label = UITK.create_fontstring(b, nil)
		b.mfac_label:SetSize(23, 16)
		b.mfac_label:SetPoint("LEFT", b.mfac_texture, "RIGHT", 3, -2)

		b.mfac_times = UITK.create_fontstring(b, nil, {JustifyH = "RIGHT"})
		b.mfac_times:SetSize(45, 16)
		b.mfac_times:SetPoint("LEFT", b.mfac_label, "RIGHT", 3, 0)

		b.mfac_rep = UITK.create_fontstring(b, nil, {JustifyH = "RIGHT"})
		b.mfac_rep:SetSize(40, 16)
		b.mfac_rep:SetPoint("TOP", b.mfac_times, "TOP", 0, 0)
		b.mfac_rep:SetPoint("RIGHT", UI.detail_slider, "LEFT", -3, 0)

		b.mfac_name = UITK.create_fontstring(b, nil)
		b.mfac_name:SetSize(250, 16)
		b.mfac_name:SetPoint("LEFT", b.mfac_times, "RIGHT", 2, 0)
		b.mfac_name:SetPoint("RIGHT", b.mfac_rep, "LEFT", 0, 0)

		UI.detail_entry_buttons[i] = b
	end


	UI.detail_options_divider = UITK.create_divider(rdf, nil);
	UI.detail_options_divider:SetPoint("TOP", UI.detail_slider_button_scroll_down, "BOTTOM", 0, 2)
	UI.detail_options_divider:SetPoint("RIGHT", UI.detail_checkbox_divider, "RIGHT", 0, 0)
	UI.detail_options_divider:SetPoint("LEFT", UI.detail_checkbox_divider, "LEFT", 0, 0)

	local checkbox_line_spacing = 6
	local TIPS = FIZ_TXT.elements.tip
	local strings_checkbox ={
		{name = "detail_show_all",			MFAC_name = "", 				text = FIZ_TXT.showAll, 		tip_title = FIZ_TXT.showAll, tip = TIPS.FIZ_ShowAllButton, relative_to = UI.detail_options_divider, anchor2 = "TOPLEFT", x = 5, y = -8},
		{name = "detail_show_quests",		MFAC_name = "ShowQuests",	text = FIZ_TXT.showQuests, 	tip_title = FIZ_TXT.showQuests, tip = TIPS.FIZ_ShowQuestButton},
		{name = "detail_show_instances", MFAC_name = "ShowInstances",text = FIZ_TXT.showInstances,tip_title = FIZ_TXT.showInstances, tip = TIPS.FIZ_ShowInstancesButton},
		{name = "detail_show_mobs", 		MFAC_name = "ShowMobs",		text = FIZ_TXT.showMobs,		tip_title = FIZ_TXT.showMobs, tip = TIPS.FIZ_ShowMobsButton},
		{name = "detail_show_items", 		MFAC_name = "ShowItems",	text = FIZ_TXT.showItems,		tip_title = FIZ_TXT.showItems, tip = TIPS.FIZ_ShowItemsButton},
		{name = "detail_general",			MFAC_name = "ShowGeneral",	text = FIZ_TXT.showGeneral,	tip_title = FIZ_TXT.showGeneral, tip = TIPS.FIZ_ShowGeneralButton},
	};

	local function checkbox_click_function(self)
		MFAC_Data[self.MFAC_name] = self:GetChecked()
		UITK.PlayClick(MFAC_Data[self.MFAC_name])
		FIZ_BuildUpdateList()
		MFAC.UpdateDetailList()
	end

	local function checkbox_all_click_function(self)
		local is_checked = self:GetChecked()
		UITK.PlayClick(is_checked)
		for _, d in ipairs({"ShowQuests", "ShowInstances", "ShowMobs", "ShowItems", "ShowGeneral"}) do
			MFAC_Data[d] = is_checked
		end

		FIZ_BuildUpdateList()
		MFAC.UpdateDetailList()
	end


	for i, v in ipairs(strings_checkbox) do
		local check_box = UITK.create_checkbox(rdf, nil, v.text, {tip_title = v.tip_title, tip = v.tip})
		UI[v.name] = check_box;
		local x, y = (v.x or 0), (v.y or checkbox_line_spacing)
		local relative_to = v.relative_to or strings_checkbox[i-1].name
		if(type(v.relative_to) == "string" or v.relative_to == nil) then
			relative_to = UI[v.relative_to or strings_checkbox[i-1].name];
		elseif(type(v.relative_to) == "table") then
			relative_to = v.relative_to
		end

		check_box:SetPoint(v.anchor1 or "TOPLEFT", relative_to, v.anchor2 or "BOTTOMLEFT", x, y)
		check_box.fnt_string:SetTextColor(1, 0.82, 0)
		check_box.MFAC_name = v.MFAC_name;
		if(v.MFAC_name == "") then
			check_box:SetScript("OnClick", checkbox_all_click_function)
		else
			check_box:SetScript("OnClick", checkbox_click_function)
		end
	end

	UI.detail_expand_button = CreateFrame("Button", nil, rdf, "SmallFontButton");
	UI.detail_expand_button:SetText(MFAC.TXT.expand_all_entries);
	UI.detail_expand_button:SetSize(143, 28);
	UI.detail_expand_button:SetPoint("TOPLEFT", UI.detail_options_divider, "BOTTOM");
	UI.detail_expand_button.tooltip_title = MFAC.TXT.expand_all_entries
	UI.detail_expand_button.tooltip_text = TIPS.FIZ_ExpandButton
	UI.detail_expand_button:SetScript("OnEnter", function(self) UITK.show_tooltip(self); end)
	UI.detail_expand_button:SetScript("OnLeave", function(_self) GameTooltip_Hide(); end)
	UI.detail_expand_button:SetScript("OnClick", function(_self) UITK.PlayClick(true); FIZ_ShowUpdateEntries(true); end)

	UI.detail_collapse_button = CreateFrame("Button", nil, rdf, "SmallFontButton");
	UI.detail_collapse_button:SetText(MFAC.TXT.collapse_all_entries);
	UI.detail_collapse_button:SetHeight(UI.detail_expand_button:GetHeight());
	UI.detail_collapse_button:SetPoint("TOPLEFT", UI.detail_expand_button, "BOTTOMLEFT", 0, -4);
	UI.detail_collapse_button:SetPoint("RIGHT", UI.detail_expand_button, "RIGHT", 0, 0);
	UI.detail_collapse_button.tooltip_title = MFAC.TXT.collapse_all_entries
	UI.detail_collapse_button.tooltip_text = TIPS.FIZ_CollapseButton
	UI.detail_collapse_button:SetScript("OnEnter", function(self) UITK.show_tooltip(self); end)
	UI.detail_collapse_button:SetScript("OnLeave", function(_self) GameTooltip_Hide(); end)
	UI.detail_collapse_button:SetScript("OnClick", function(_self) UITK.PlayClick(true); FIZ_ShowUpdateEntries(false); end)

	UI.detail_clear_session_button = CreateFrame("Button", nil, rdf, "SmallFontButton");
	UI.detail_clear_session_button:SetText(MFAC.TXT.clear_session_gain);
	UI.detail_clear_session_button:SetHeight(UI.detail_expand_button:GetHeight());
	UI.detail_clear_session_button:SetPoint("TOPLEFT", UI.detail_collapse_button, "BOTTOMLEFT", 0, -4);
	UI.detail_clear_session_button:SetPoint("RIGHT", UI.detail_collapse_button, "RIGHT", 0, 0);
	UI.detail_clear_session_button.tooltip_title = MFAC.TXT.clear_session_gain
	UI.detail_clear_session_button.tooltip_text = TIPS.FIZ_ClearSessionGainButton
	UI.detail_clear_session_button:SetScript("OnEnter", function(self) UITK.show_tooltip(self); end)
	UI.detail_clear_session_button:SetScript("OnLeave", function(_self) GameTooltip_Hide(); end)
	UI.detail_clear_session_button:SetScript("OnClick", function(_self) UITK.PlayClick(true); FIZ_ClearSessionGain(); end)

	UI.detail_suppress_hint_string = UITK.create_fontstring(rdf, nil)
	UI.detail_suppress_hint_string:SetPoint("TOPLEFT", UI.detail_clear_session_button, "BOTTOMLEFT", 0, -4)
	UI.detail_suppress_hint_string:SetPoint("RIGHT", UI.detail_clear_session_button, "RIGHT", 40, 0);
	UI.detail_suppress_hint_string:SetHeight(32);
	UI.detail_suppress_hint_string:SetText(FIZ_TXT.suppressHint)

end
