local ADDON_NAME, MFAC = ... -- Pulls back the Addon-Local Variables and store them locally.
local UI = MFAC.UI
local UITK = MFAC.UITK

local TIPS = FIZ_TXT.elements.tip
local options_source

local options_frame_coords = {}

if (WOW_PROJECT_ID == WOW_PROJECT_CLASSIC) then
options_source =
{
	{name = "options_show_missing_rep_cb",			text = FIZ_TXT.showMissing, indent = 1, tip = TIPS.FIZ_EnableMissingBox, var = "ShowMissing"},
	{name = "options_show_rep_detail_frame_cb",	text = FIZ_TXT.extendDetails, indent = 1, tip = TIPS.FIZ_ExtendDetailsBox, var = "ExtendDetails"},
	{name = "options_write_detailed_rep_gain_cb",	text = FIZ_TXT.gainToChat, indent = 1, tip = TIPS.FIZ_GainToChatBox, var = "WriteChatMessage"},
	{name = "options_suppress_orig_gain_msg_cb",	text = FIZ_TXT.suppressOriginalGain, indent = 1, tip = TIPS.FIZ_SupressOriginalGainBox, var = "SuppressOriginalChat"},
	{name = "options_show_rep_preview_cb",	text = FIZ_TXT.showPreviewRep, indent = 1, tip = TIPS.FIZ_ShowPreviewRepBox, var = "ShowPreviewRep"},
	{name = "options_switch_faction_bar_cb",	text = FIZ_TXT.switchFactionBar, indent = 1, tip = TIPS.FIZ_SwitchFactionBarBox, var = "SwitchFactionBar"},
	{name = "options_silent_switch_cb",	text = FIZ_TXT.silentSwitch, indent = 2, tip = TIPS.FIZ_SilentSwitchBox, var = "SilentSwitch"},
	{name = "options_auto_turn_in_cb",	text = FIZ_TXT.AutoTurnIn, indent = 1, tip = TIPS.MFAC_AutoTurnInBox, var = "AutoTurnIn"},
}
options_frame_coords.x = -34
options_frame_coords.y = -11

else

options_source =
{
	{name = "options_show_missing_rep_cb",			text = FIZ_TXT.showMissing, indent = 1, tip = TIPS.FIZ_EnableMissingBox, var = "ShowMissing"},
	{name = "options_show_rep_detail_frame_cb",	text = FIZ_TXT.extendDetails, indent = 1, tip = TIPS.FIZ_ExtendDetailsBox, var = "ExtendDetails"},
	{name = "options_write_detailed_rep_gain_cb",	text = FIZ_TXT.gainToChat, indent = 1, tip = TIPS.FIZ_GainToChatBox, var = "WriteChatMessage"},
	{name = "options_no_guild_gain_msg_cb",	text = FIZ_TXT.noGuildGain, indent = 2, tip = TIPS.FIZ_NoGuildGainBox, var = "NoGuildGain"},
	{name = "options_suppress_orig_gain_msg_cb",	text = FIZ_TXT.suppressOriginalGain, indent = 1, tip = TIPS.FIZ_SupressOriginalGainBox, var = "SuppressOriginalChat"},
	{name = "options_show_rep_preview_cb",	text = FIZ_TXT.showPreviewRep, indent = 1, tip = TIPS.FIZ_ShowPreviewRepBox, var = "ShowPreviewRep"},
	{name = "options_switch_faction_bar_cb",	text = FIZ_TXT.switchFactionBar, indent = 1, tip = TIPS.FIZ_SwitchFactionBarBox, var = "SwitchFactionBar"},
	{name = "options_silent_switch_cb",	text = FIZ_TXT.silentSwitch, indent = 2, tip = TIPS.FIZ_SilentSwitchBox, var = "SilentSwitch"},
	{name = "options_no_guild_switch_cb",	text = FIZ_TXT.noGuildSwitch, indent = 2, tip = TIPS.FIZ_NoGuildSwitchBox, var = "NoGuildSwitch"},
	{name = "options_show_paragon_standing_cb",	text = FIZ_TXT.showParagon, indent = 1, tip = TIPS.FIZ_EnableParagonBox, var = "ShowParagon"},
	{name = "options_show_toasts_cb",	text = FIZ_TXT.ShowToasts, indent = 1, tip = TIPS.FIZ_ShowToastsBox, var = "ShowToasts"},
	{name = "options_fade_bags_cb",	text = MFAC.TXT.fade_bags_label, indent = 1, tip = MFAC.TXT.fade_bags_tip, var = "FadeParagonBags"},
	{name = "options_auto_turn_in_cb",	text = FIZ_TXT.AutoTurnIn, indent = 1, tip = TIPS.MFAC_AutoTurnInBox, var = "AutoTurnIn"},
}

options_frame_coords.x = 2
options_frame_coords.y = 0
end


function MFAC.build_options_frame()
	UI.ReputationOptionsFrame = CreateFrame("FRAME", "MFAC_ReputationOptionsFrame", ReputationFrame, BackdropTemplateMixin and "BackdropTemplate")
	local rof = UI.ReputationOptionsFrame
	rof:EnableMouse(true)
	rof:SetBackdrop({
		bgFile = "Interface/ChatFrame/ChatFrameBackground",
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true,
		tileSize = 32,
		edgeSize = 16,
		insets = { left = 6, right = 6, top = 4, bottom = 4 }
	})
	rof:SetBackdropColor(0, 0, 0, 0.97);
	rof:SetPoint("TOPLEFT", ReputationFrame, "TOPRIGHT", options_frame_coords.x, options_frame_coords.y)
	rof:SetSize(450, 570);
	rof:Hide();

	UI.ReputationOptionsFrame:SetScript("OnShow", function(_self, _button, _down)
		for _i, v in ipairs(options_source) do
			UI[v.name]:SetChecked(MFAC_Data[v.var]);
		end
	end)

	---------------
	-- Top bar
	---------------
	UI.option_title = UITK.create_fontstring(rof, nil)
	UI.option_title:SetTextColor(1, 0.82, 0)
	UI.option_title:SetFont(UITK.default_font_name, 13, "OUTLINE")
	UI.option_title:SetPoint("TOP", rof, "TOP", 0, -8)
	local this_version = GetAddOnMetadata(ADDON_NAME, "Version")
	UI.option_title:SetText(ADDON_NAME .. " " .. FIZ_TXT.options .. " v" .. this_version)

	UI.option_close_button = CreateFrame("Button", nil, rof, "UIPanelCloseButton")
	UI.option_close_button:SetPoint("TOPRIGHT", rof, "TOPRIGHT", 3, 2)
	UI.option_close_button:SetScript("OnClick", function(self, _button, _down)
		UITK.PlayClick(true)
		HideParentPanel(self);
	end)

	UI.option_title_divider = UITK.create_divider(rof, nil);
	UI.option_title_divider:SetPoint("TOPLEFT", 5, -19)
	UI.option_title_divider:SetPoint("RIGHT", rof, "RIGHT", -4, 0)

	local line_spacing = -3

	rof:SetHeight(#options_source * (26 - line_spacing) + 50);

	for i, v in ipairs(options_source) do
		UI[v.name] = UITK.create_checkbox(rof, nil, v.text, {tip_title = v.text, tip = v.tip or ""})
		UI[v.name].fnt_string:SetTextColor(1, 0.82, 0)
		if(i == 1) then
			UI[v.name]:SetPoint("TOP", UI.option_title_divider, "BOTTOM", 0, line_spacing)
		else
			UI[v.name]:SetPoint("TOP", UI[options_source[i - 1].name], "BOTTOM", 0, line_spacing)
		end
		UI[v.name]:SetPoint("LEFT", rof, "LEFT", v.indent * 20 - 5, 0)
		UI[v.name].mfac_var = v.var
		UI[v.name]:SetScript("OnClick", function(self)
			MFAC_Data[self.mfac_var] = self:GetChecked()
			if ( self:GetChecked() ) then
				PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON);
			else
				PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_OFF);
			end
			ReputationFrame_Update()
		end)

	end

	local muffin_texture = rof:CreateTexture("MuffinWhatsNewFrameMuffinTexture", "ARTWORK");
	muffin_texture:SetPoint("BOTTOMRIGHT", rof, "BOTTOMRIGHT", -2, 5)
	muffin_texture:SetSize(48, 48);
	muffin_texture:SetTexture("Interface\\Addons\\" .. ADDON_NAME .. "\\Textures\\muffin.tga")
	muffin_texture:SetBlendMode("BLEND")

end
