local _, MFAC = ... -- Pulls back the Addon-Local Variables and store them locally.


function MFAC.get_npc_id(p_guid)

	if(p_guid == nil) then
		--TODO: Raise error
		return nil;
	end

	local _type, _zero, _server_id, _instance_id, _zone_uid, npc_id, _spawn_uid = strsplit("-", p_guid);

	if (_type == "Creature") then
		return tonumber(npc_id);
	end

	return nil;
end

function MFAC.Print(msg, forceDefault)
	if not (msg) then return end

	if ((MFAC_Data == nil) or forceDefault) then
		DEFAULT_CHAT_FRAME:AddMessage(msg)
	else
		for i = 1, NUM_CHAT_WINDOWS do
			local chatTab = _G["ChatFrame"..i.."Tab"]
			if chatTab:IsShown() then
				local chatFrame = _G["ChatFrame"..i]
				local messageTypes = chatFrame.messageTypeList
				for j = 1, #messageTypes do
					if messageTypes[j] == "COMBAT_FACTION_CHANGE" then
						_G["ChatFrame"..i]:AddMessage(msg)
					end
				end
			end
		end

	end
end

--
-- Print a message, but only in Debug Mode
--
function MFAC.DebugPrint(...)
	if (MFAC.DebugEnabled) then
		print(...)
	end
end

--
--	Convert a Boolean value to an enabled/disabled string
--
function MFAC.BoolToEnabled(b)
	local result = MFAC.Colours.Disabled .. FIZ_TXT.disabled .. "|r";
	if b then
		result = MFAC.Colours.Enabled .. FIZ_TXT.enabled .. "|r";
	end
	return result
end

function MFAC.ValidateFromTo(p_from, p_to, p_stack_trace)
	local success = 1

	if((p_from == nil) or (type(p_from) ~= "number") or (p_from < 1) or (p_from > 8) or (p_from > p_to) ) then
		print("      Invalid From:", p_from, string.match(p_stack_trace, "(%S+.lua:%d+)"));
		success = 0;
	end

	if((p_to == nil) or (type(p_to) ~= "number") or (p_to < 1) or (p_to > 8) ) then
		print("      Invalid To:", p_to, string.match(p_stack_trace, "(%S+.lua:%d+)"));
		success = 0;
	end

	return success;
end

local MFACScanningTooltip = CreateFrame("GameTooltip", "MFACScanningTooltip", UIParent, "GameTooltipTemplate")
-- luacheck: globals MFACScanningTooltipTextLeft1
function MFAC.GetQuestNameForID(p_quest_id)

	if(MFAC_Data.quest_names == nil) then
		MFAC_Data.quest_names = {}
	end

	local quest_name;

	quest_name = MFAC_Data.quest_names[p_quest_id]

	if(not quest_name or quest_name == RETRIEVING_DATA) then
		quest_name = C_QuestLog.GetTitleForQuestID(p_quest_id)
	end

	if((not quest_name or quest_name == RETRIEVING_DATA) and C_TaskQuest) then	--C_TaskQuest doesn't exist in Classic
		quest_name = C_TaskQuest.GetQuestInfoByQuestID(p_quest_id)
	end

	-- No luck, try the tooltip
	if( (WOW_PROJECT_ID ~= WOW_PROJECT_CLASSIC) and (not quest_name or quest_name == RETRIEVING_DATA)) then
		MFACScanningTooltip:SetOwner(UIParent, "ANCHOR_NONE")
		MFACScanningTooltip:SetHyperlink("quest:" .. p_quest_id)
		quest_name = MFACScanningTooltipTextLeft1:GetText()
		MFACScanningTooltip:Hide()
	end

	--Cache it and return it if it's valid
	if(quest_name and quest_name ~= RETRIEVING_DATA) then
		MFAC_Data.quest_names[p_quest_id] = quest_name
		return MFAC_Data.quest_names[p_quest_id]
	end

	-- We hadn't cached it and couldn't read it, so mark it as existing but unknown
	MFAC_Data.quest_names[p_quest_id] = false

	return p_quest_id;

end

function MFAC.is_table_empty(p_table)

	return (next(p_table) == nil)

end

function MFAC.table_size(p_table)
	local result = 0
	if p_table then
		for _item in pairs(p_table) do
			result = result + 1;
		end
	end
	return result
end

function MFAC.get_bar_values_for_faction(p_faction_info)
	local faction = p_faction_info.faction;
	local friend = p_faction_info.friend;
	local paragon = p_faction_info.paragon;

	local bar_min, bar_max, bar_value = faction.bar_min, faction.bar_max, faction.bar_value

	-- If it's a Friend, ignore faction numbers and look at friend_info
	if (friend.friend_id ~= nil) then
		if ( friend.next_threshold ) then
			bar_min, bar_max, bar_value = friend.threshold, friend.next_threshold, friend.current_rep;
		else
			-- max rank, make it look like a full bar
			bar_min, bar_max, bar_value = 0, 1, 1;
		end
	end

	-- If we're at max reputation rank (Exalted), show it as a full bar
	if(faction.standing_id == MAX_REPUTATION_REACTION) then
		bar_min, bar_max, bar_value = 0, 1, 1;
	end

	-- If it's a Paragon rep, replace previous values with Paragon ones
	if(faction.faction_id and (paragon.current_value ~= nil) and MFAC_Data.ShowParagon) then
		bar_min, bar_max, bar_value = 0, paragon.threshold, (paragon.current_value % paragon.threshold);
	end

	--Normalize Values
	bar_max = bar_max - bar_min;
	bar_value = bar_value - bar_min;

	-- After normalization, bar_min will always be 0, so we don't need to return it
	return bar_max, bar_value
end

function MFAC.get_faction_standing(p_faction_info, p_gender)

	local standing_text
	local standing_id = p_faction_info.faction.standing_id;
	local colour_index = standing_id

	if (p_faction_info.friend.standing_text) then
		standing_text = p_faction_info.friend.standing_text;
		colour_index = 5
	elseif (standing_id == 9) then
		standing_text = "Paragon";	-- TODO: Localize!!
	else
		standing_text = GetText("FACTION_STANDING_LABEL" .. standing_id, p_gender);
	end

	return standing_text, colour_index;
end

function MFAC.RGBColourString(a, r, g, b)
	return string.format("|c%02X%02X%02X%02X", a*255, r*255, g*255, b*255)
end

--luacheck: ignore 111/dump 113/dump
function MFAC.dump(o, indent)
	indent = indent or 0
	local ind = (strrep(" ", indent * 3))
	if type(o) == 'table' and (indent < 3) then
	   local s = ind .. '{ '
	   for k,v in pairs(o) do
		  if type(k) ~= 'number' then k = '"'..k..'"' end
		  s = s .. '['..k..'] = ' .. dump(v, indent +1) .. ','
	   end
	   return s .. '}|n'
	else
	   return tostring(o)
	end
 end

function MFAC.GetNumQuestLogEntries()

	if (C_QuestLog and C_QuestLog.GetNumQuestLogEntries) then
		return C_QuestLog.GetNumQuestLogEntries()
	else
		return GetNumQuestLogEntries()
	end
end

function MFAC.GetQuestInfoForIndex(p_quest_idx)

	if (C_QuestLog and C_QuestLog.GetInfo) then
		local tmp = C_QuestLog.GetInfo(p_quest_idx)
		local complete = C_QuestLog.IsComplete(tmp.questID)
		return tmp.title, tmp.isHeader, complete
	else
		local title, _level, _tag, _group, header, _collapsed, complete, _daily = GetQuestLogTitle(z)
		return title, header, complete
	end
end



