
local _ADDON_NAME, MFAC = ... -- Pulls back the Addon-Local Variables and store them locally.


function MFAC.IsFactionParagon(_p_faction)
	return false;
end

function MFAC.GetFactionParagonInfo(_p_faction_id)
	return nil;
end

function MFAC.ShowReputationBarHighlight(p_rep_bar)

	_G["ReputationBar" .. p_rep_bar .. "Highlight1"]:Show();
	_G["ReputationBar" .. p_rep_bar .. "Highlight2"]:Show();

end

function MFAC.HideReputationBarHighlight(p_rep_bar)

	_G["ReputationBar" .. p_rep_bar .. "Highlight1"]:Hide();
	_G["ReputationBar" .. p_rep_bar .. "Highlight2"]:Hide();

end
function MFAC.ExtractSkills()

	local fishing_name = GetSpellInfo(7620)
	local cooking_name = GetSpellInfo(2550)
	local firstaid_name = GetSpellInfo(3273)

	local limit = MFAC.LimitType;
	MFAC.SkillTrackBits = 0;

	local num_skills = GetNumSkillLines();
	for i=1, num_skills() do
		local skill_name = GetSkillLineInfo(i)

		if(skill_name == fishing_name) then
			MFAC.SkillTrackBits = MFAC.SkillTrackBits + limit.Fishing;
		end
		if(skill_name == cooking_name) then
			MFAC.SkillTrackBits = MFAC.SkillTrackBits + limit.Cooking;
		end
		if(skill_name == firstaid_name) then
			MFAC.SkillTrackBits = MFAC.SkillTrackBits + limit.FirstAid;
		end
	end


end

function MFAC.StatusTrackingBarManager_UpdateBarsShown()

	--NOP shim since this call is needed in Retail

end

function MFAC.ReputationFrame_Update()

	local numFactions
	if MFAC_Data.SortByStanding then
		FIZ_StandingSort()
		numFactions = FIZ_OBS_numFactions
	else
		numFactions = GetNumFactions();
	end

	-- Update scroll frame
	if ( not FauxScrollFrame_Update(ReputationListScrollFrame, numFactions, NUM_FACTIONS_DISPLAYED, REPUTATIONFRAME_FACTIONHEIGHT ) ) then
		ReputationListScrollFrameScrollBar:SetValue(0);
	end
	local factionOffset = FauxScrollFrame_GetOffset(ReputationListScrollFrame);

	-- Set "Standing" title on Reputation frame
	if (MFAC_Data.ShowMissing) then
		ReputationFrameStandingLabel:SetText(MFAC.original_standing_text .. " " .. FIZ_TXT.missing)
	else
		ReputationFrameStandingLabel:SetText(MFAC.original_standing_text)
	end

	if (MFAC_Data.ShowPreviewRep) then
		FIZ_ParseBagContent()
	end

	for i = 1, NUM_FACTIONS_DISPLAYED, 1 do
		local factionIndex = factionOffset + i;
		local factionBar = _G["ReputationBar"..i];
		local factionHeader = _G["ReputationHeader"..i];
		local factionCheck = _G["ReputationBar"..i.."Check"]
		local factionName = _G["ReputationBar"..i.."FactionName"];
		local factionStanding = _G["ReputationBar"..i.."FactionStanding"];
		local factionRightBarTexture = _G["ReputationBar"..i.."ReputationBarRight"]
		local factionAtWarIndicator = _G["ReputationBar"..i.."AtWarCheck"]

		--local factionBarPreview = _G["FIZ_StatusBar"..i];

		if ( factionIndex <= numFactions ) then
			if MFAC_Data.SortByStanding then
				MFAC.SortByStanding(i, factionIndex, factionBar, factionBarPreview, factionCheck, factionName, factionHeader, factionStanding, factionAtWarIndicator, factionRightBarTexture)
			else
				MFAC.OriginalRepOrder(i, factionIndex, factionBar, factionBarPreview, factionCheck, factionName, factionHeader, factionStanding, factionAtWarIndicator, factionRightBarTexture)
			end
		else
			factionBar:Hide();
			factionHeader:Hide()
		end
	end

	if ( GetSelectedFaction() == 0 ) then
		ReputationDetailFrame:Hide();
		MFAC.UI.ReputationDetailFrame:Hide();
	end

end

function MFAC.SortByStanding(i, factionIndex, factionRow, factionBar, factionBarPreview, factionCheck, factionTitle, factionButton, factionStanding, factionBackground)

	local OBS_fi = FIZ_Entries[factionIndex]
	local OBS_fi_i = OBS_fi.i

	if (OBS_fi.header) then
		FIZ_ReputationFrame_SetRowType(factionRow, isChild, OBS_fi.header, hasRep);
		-- display the standingID as Header
		if (OBS_fi_i == 9) then
			factionTitle:SetText("Paragon".." ("..tostring(OBS_fi.size)..")");
		elseif (OBS_fi_i == 8) then
			factionTitle:SetText(GetText("FACTION_STANDING_LABEL"..OBS_fi_i, gender).." ("..tostring(OBS_fi.size)..")");
		else
			factionTitle:SetText(GetText("FACTION_STANDING_LABEL"..OBS_fi_i, gender).." -> "..GetText("FACTION_STANDING_LABEL"..OBS_fi_i+1, gender).." ("..tostring(OBS_fi.size)..")");
		end
		if ( FIZ_Collapsed[OBS_fi_i] ) then
			factionButton:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
		else
			factionButton:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up");
		end
		factionRow.index = factionIndex
		factionRow.isCollapsed = FIZ_Collapsed[OBS_fi_i];
	else
		-- get the info for this Faction
		local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild, factionID, hasBonusRepGain = GetFactionInfo(OBS_fi_i);
		factionTitle:SetText(name);
		local colorIndex, isCappedFriendship, factionStandingtext  = FIZ_Friend_Detail(factionID, standingID, factionRow);
		local current_paragon, paragon_max, paragon_reward_quest_id, paragon_reward_pending

		if(factionID) then
			current_paragon, paragon_max, paragon_reward_quest_id, paragon_reward_pending = MFAC.GetFactionParagonInfo(factionID);
		end

		local origBarValue = barValue

		-- For Paragon-capable Factions, show the Paragon rep icon
		if ( factionID and MFAC.IsFactionParagon(factionID) ) then
			local paragonFrame = ReputationFrame.paragonFramesPool:Acquire();
			paragonFrame.factionID = factionID;
			paragonFrame:SetPoint("RIGHT", factionRow, 11, 0);
			origBarValue = current_paragon % paragon_max;
			C_Reputation.RequestFactionParagonPreloadRewardData(factionID);
			paragonFrame.Glow:SetShown(paragon_reward_pending);
			paragonFrame.Check:SetShown(paragon_reward_pending);
			paragonFrame:Show();
		end

		local isCapped = (standingID == MAX_REPUTATION_REACTION);

-- Set reputation bar to paragon values if user option is activated and faction is at paragon rep
		if(factionID and current_paragon and MFAC_Data.ShowParagon) then
			barMin, barMax, barValue = 0, paragon_max, current_paragon % paragon_max;
		elseif(standingID == 8 or isCappedFriendship) then
			barMin, barMax, barValue = 0, 1, 1;
		end
		--TODO: Clean this up. Use my normalize and this is also duplicated in FIZ_OriginalRepOrder
		-- Normalize Values
		barMax = barMax - barMin;
		barValue = barValue - barMin;
		barMin = 0;

		if (MFAC_Data.ShowMissing) then
			factionRow.rolloverText = nil;
		elseif (not isCapped and not isCappedFriendship) then
			factionRow.rolloverText = HIGHLIGHT_FONT_COLOR_CODE .. " " .. format(REPUTATION_PROGRESS_FORMAT, BreakUpLargeNumbers(barValue), BreakUpLargeNumbers(barMax)) .. FONT_COLOR_CODE_CLOSE;
		elseif (current_paragon and MFAC_Data.ShowParagon) then
			factionRow.rolloverText = HIGHLIGHT_FONT_COLOR_CODE .. " " .. format(REPUTATION_PROGRESS_FORMAT, BreakUpLargeNumbers(barValue), BreakUpLargeNumbers(barMax)) .. FONT_COLOR_CODE_CLOSE;
		else
			factionRow.rolloverText = nil;
		end

		local toExalted = 0
		if (standingID < MAX_REPUTATION_REACTION) then
			toExalted = MFAC.ToExalted[standingID] + barMax - barValue;
		end

		factionRow.index = OBS_fi_i;

		local missing_text = ""
		if (MFAC_Data.ShowMissing) then
			missing_text = " (" .. BreakUpLargeNumbers(barMax - barValue) .. ")";
		end

		if(current_paragon and MFAC_Data.ShowParagon) then
			factionStandingtext = "Paragon";
		end

		factionRow.standingText = factionStandingtext .. missing_text;

		factionStanding:SetText(factionRow.standingText);

		if ( isCappedFriendship ) then
			factionRow.tooltip = nil;
		else
			factionRow.tooltip = HIGHLIGHT_FONT_COLOR_CODE.." "..barValue.." / "..barMax..FONT_COLOR_CODE_CLOSE;
		end

		factionBar:SetMinMaxValues(0, barMax);
		factionBar:SetValue(barValue);
		local color = FACTION_BAR_COLORS[standingID];
		factionBar:SetStatusBarColor(color.r, color.g, color.b);


		local previewValue = 0
		if (factionBarPreview and MFAC_Data.ShowPreviewRep) then
			previewValue = FIZ_GetReadyReputation(OBS_fi_i)
		end
		if ((previewValue > 0) and not ((standingID==8) and (barMax-barValue == 1) ) ) then
			factionBarPreview:Show()
			factionBarPreview:SetMinMaxValues(0, barMax)
			previewValue = previewValue + barValue
			if (previewValue > barMax) then previewValue = barMax end
			factionBarPreview:SetValue(previewValue)
			factionBarPreview:SetID(factionIndex)
			factionBarPreview:SetStatusBarColor(0.8, 0.8, 0.8, 0.5)
		else
			factionBarPreview:Hide()
		end

		FIZ_ReputationFrame_SetRowType(factionRow, isChild, OBS_fi.header, hasRep);
		factionRow:Show();
		-- Update details if this is the selected Faction
		if ( atWarWith ) then
			_G["ReputationBar"..i.."ReputationBarAtWarHighlight1"]:Show();
			_G["ReputationBar"..i.."ReputationBarAtWarHighlight2"]:Show();
		else
			_G["ReputationBar"..i.."ReputationBarAtWarHighlight1"]:Hide();
			_G["ReputationBar"..i.."ReputationBarAtWarHighlight2"]:Hide();
		end
		-- Update details if this is the selected faction
		if ( OBS_fi_i == GetSelectedFaction() ) then
			if ( ReputationDetailFrame:IsShown() ) then
				FIZ_ReputationDetailFrame_IsShown(OBS_fi_i, i)
			end

			if ( MFAC.UI.ReputationDetailFrame:IsVisible() ) then
				FIZ_Rep_Detail_Frame(OBS_fi_i,standingID,barValue,barMax,origBarValue,standingID,toExalted,factionStandingtext)
			end

			MFAC.ShowReputationBarHighlight(i)
		else
			MFAC.HideReputationBarHighlight(i)
		end
	end

end

function MFAC.OriginalRepOrder(i, factionIndex, factionBar, factionBarPreview, factionCheck, factionName, factionHeader, factionStanding, factionAtWarIndicator, factionRightBarTexture)

	local gender = UnitSex("player");
	local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild, factionID, hasBonusRepGain = GetFactionInfo(factionIndex);

	if(isHeader) then
		factionHeader.Text:SetText(name)

		if ( isCollapsed ) then
			factionHeader:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
		else
			factionHeader:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up");
		end

		factionHeader.index = factionIndex;
		factionHeader.isCollapsed = isCollapsed;

		factionBar:Hide()
		factionHeader:Show()
		factionCheck:Hide()
		return
	end

	factionBar.hasRep = (hasRep) or (not isHeader)

	local isCapped = (standingID == MAX_REPUTATION_REACTION);

	local origBarValue = barValue

	local faction_standing_text = GetText("FACTION_STANDING_LABEL" .. standingID, gender);
	factionName:SetText(name);

	if (atWarWith) then
		factionAtWarIndicator:Show()
	else
		factionAtWarIndicator:Hide()
	end

-- If exalted show a full green bar
	if(standingID == 8) then
		barMin, barMax, barValue = 0, 1, 1;
	end

	--TODO: Clean this up, use my normalize function
	-- Normalize Values
	barMax = barMax - barMin;
	barValue = barValue - barMin;
	barMin = 0;

	factionBar.index = factionIndex;
	--TODO: This code is duplicated, fix this

	if (MFAC_Data.ShowMissing) then
		factionBar.standingText = faction_standing_text .. " (" .. FormatLargeNumber(barMax - barValue) .. ")";
	else
		factionBar.standingText = faction_standing_text;
	end

	factionStanding:SetText(factionBar.standingText);
	factionBar.tooltip = HIGHLIGHT_FONT_COLOR_CODE .. " " .. barValue .. " / " .. barMax .. FONT_COLOR_CODE_CLOSE;
	factionBar:SetMinMaxValues(0, barMax);
	factionBar:SetValue(barValue);
	local color = FACTION_BAR_COLORS[standingID];
	factionBar:SetStatusBarColor(color.r, color.g, color.b);
	factionBar:SetID(factionIndex);
	factionBar:Show();
	factionHeader:Hide();

	-- Show a checkmark if this faction is being watched
	if ( isWatched ) then
		factionCheck:Show();
		factionName:SetWidth(100);
		factionCheck:SetPoint("LEFT", factionName, "LEFT", factionName:GetStringWidth(), 0);
	else
		factionCheck:Hide();
		factionName:SetWidth(110);
	end


	if (MFAC_Data.ShowMissing or isCapped) then
		factionBar.rolloverText = nil;
	else
		factionBar.rolloverText = HIGHLIGHT_FONT_COLOR_CODE .. " " .. format(REPUTATION_PROGRESS_FORMAT, BreakUpLargeNumbers(barValue), BreakUpLargeNumbers(barMax)) .. FONT_COLOR_CODE_CLOSE;
	end



	if ( factionIndex == GetSelectedFaction() ) then

		local toExalted = 0
		if (standingID < MAX_REPUTATION_REACTION) then
			toExalted = MFAC.ToExalted[standingID] + barMax - barValue;
		end

		if ( ReputationDetailFrame:IsShown() ) then
			FIZ_ReputationDetailFrame_IsShown(factionIndex, i)
		end

		if ( MFAC.UI.ReputationDetailFrame:IsVisible() ) then
			FIZ_Rep_Detail_Frame(factionIndex, standingID, barValue, barMax, origBarValue, standingID, toExalted, faction_standing_text)
			MFAC.ShowReputationBarHighlight(i)
		end
	else
		MFAC.HideReputationBarHighlight(i)
	end
end
